----------------------------------------------------------------
-- Archivo de creacion de tablas para Azucar CMS              --
-- Fecha: Febrero 01 de 2005                                 --
-- Ultima modificacion:                                       --
-- Autor: Camilo Torres C.                                    --
----------------------------------------------------------------

-------------------------------
-- Clases Objeto del sistema --
-------------------------------

-- Objetos protegidos
CREATE TABLE sis_objetos_protegidos ( 
    id_objeto SERIAL,
    id_contenedor INTEGER,
    id_modulo INTEGER,
    publico INTEGER DEFAULT 1,
    PRIMARY KEY(id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES sis_objetos_protegidos (id_objeto)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_objetos_protegidos IS 'Todo objeto sobre el cual un usuario debe tener un rol para poder manipularlo. Es una super clase abstracta';   
COMMENT ON COLUMN sis_objetos_protegidos.id_objeto IS 'Identificador único de un objeto'; 
COMMENT ON COLUMN sis_objetos_protegidos.id_contenedor IS 'Identificador del objeto que contiene a este, representa la jerarquia de contenimiento'; 
COMMENT ON COLUMN sis_objetos_protegidos.id_modulo IS 'Identifica que modulo tiene a este objeto'; 
COMMENT ON COLUMN sis_objetos_protegidos.publico IS 'Dice si el objeto es publico o protegido'; 


-- Tipos de identificacion
CREATE TABLE sis_tipo_identificacion (
    nombre VARCHAR(50) NOT NULL,
    PRIMARY KEY (nombre)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_tipo_identificacion IS 'Tipos de identificación para un contacto, como cc, ce, cm.';   
COMMENT ON COLUMN sis_tipo_identificacion.nombre IS 'Nombre del tipo de identificacion'; 



-- Contactos Generales
CREATE TABLE sis_contactos (
    id_contacto serial,
    apellido1 VARCHAR(50),
    apellido2 VARCHAR(50),
    tipo_identificacion VARCHAR(50),
    email VARCHAR(50),
    celular VARCHAR(50),
    sexo character(1),
    nombres VARCHAR(100),
    cargo VARCHAR(60),
    telefono1 VARCHAR(50),
    telefono2 VARCHAR(50),
    empresa VARCHAR(150),
    identificacion VARCHAR(50),
    PRIMARY KEY(id_contacto),
    FOREIGN KEY (tipo_identificacion) REFERENCES sis_tipo_identificacion (nombre)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_contactos IS 'Información detallada de un usuario; son los datos generales';   


CREATE TABLE sis_usuarios (
    nombre VARCHAR(255),
    password VARCHAR(100),
    pregunta TEXT,
    respuesta TEXT,
    id_contacto INTEGER,
    PRIMARY KEY(id_objeto),
    FOREIGN KEY (id_contacto) REFERENCES sis_contactos (id_contacto)
    -- Asociacion id_contendor ==> TRIGGER (Clase abstracta Sis_sitios)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_usuarios IS 'Usuarios que requieren autenticacion para usar el sistema, tanto como adimistrador como navegante';   


CREATE TABLE sis_grupos (
    nombre VARCHAR(255),
    PRIMARY KEY (id_objeto)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_grupos IS 'Agrupación de usuarios que comparten permisos sobre objetos';   


CREATE TABLE sis_grupos_usuarios (
    id_usuario INTEGER,
    id_grupo INTEGER,
    FOREIGN KEY (id_usuario) REFERENCES sis_usuarios (id_objeto)  ON DELETE CASCADE,
    FOREIGN KEY (id_grupo) REFERENCES sis_grupos (id_objeto)    
);
-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_grupos_usuarios IS 'Producto cartesiano entre grupos y usuarios (Relación muchos a muchos)';   


CREATE TABLE sis_roles (
    id_rol SERIAL,
    tipo INTEGER DEFAULT 0,
    id_usuario INTEGER,
    id_objeto INTEGER NOT NULL,
    PRIMARY KEY (id_rol)
    -- Asociación de id_usuario con el id_grupo de sis_grupos o id_objeto de sis_usuarios ==> TRIGGER (asociación disyunta)
    -- Asociacion con sis_objetos_protegidos ==> TRIGGER (clase abstracta)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_roles IS 'Permisos de un usuario sobre un objeto'; 


CREATE TABLE sis_estilos_textos (
    id_estilo_texto SERIAL,
    fuente VARCHAR(32),
    color_fuente VARCHAR(32),
    tamagno INTEGER,
    color_fondo VARCHAR(32),
    borde VARCHAR(32),
    PRIMARY KEY (id_estilo_texto)
);

CREATE TABLE sis_hojas_estilos (
    h1 INTEGER,
    h2 INTEGER,
    h3 INTEGER,
    h4 INTEGER,
    body INTEGER,
    a INTEGER,
    th INTEGER,
    tr_par INTEGER,
    tr_impar INTEGER,
    tabla_formulario INTEGER,
    boton INTEGER,
    menu INTEGER,
    menu_over INTEGER,
    cabecera INTEGER,
    nombre VARCHAR(255),
    PRIMARY KEY (id_objeto),
    -- Cada objeto de la hoja de estilos es de tipo estilo texto
    FOREIGN KEY (h1) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (h2) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (h3) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (h4) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (body) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (a) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (th) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (tr_par) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (tr_impar) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (tabla_formulario) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (boton) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (menu) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (menu_over) REFERENCES sis_estilos_textos(id_estilo_texto)
) INHERITS (sis_objetos_protegidos);

CREATE TABLE sis_sitios (
    url VARCHAR(50),
    id_hoja_estilo INTEGER,
    num_usuarios INTEGER,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_hoja_estilo) REFERENCES sis_hojas_estilos(id_objeto)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_sitios IS 'Sitio que esta siendo administrado con CMS'; 


CREATE TABLE sis_idiomas (
    id_idioma SERIAL,
    nombre VARCHAR(50),
    bandera VARCHAR(50),
    enctype CHAR(5),
    PRIMARY KEY (id_idioma)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_idiomas IS 'Idiomas en los que se alimenta sitios de CMS'; 


CREATE TABLE sis_estilos_bodys (
    scroll_fondo VARCHAR(32),
    scroll_barra VARCHAR(32),
    scroll_flecha VARCHAR(32),
    PRIMARY KEY (id_estilo_texto)
) INHERITS (sis_estilos_textos);

CREATE TABLE sis_paginas (
    id_estilo INTEGER,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_estilo) REFERENCES sis_hojas_estilos(id_objeto)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_paginas IS 'Pagina es todo objeto que es enlazable desde el menu.'; 


-- Tabla para los tipos de modulos
CREATE TABLE sis_tipo_modulo (
    id_tipo SERIAL,
    nombre VARCHAR(50),
    descripcion VARCHAR(255),
    carpeta VARCHAR(50),
    prefijo CHAR(3),
    PRIMARY KEY (id_tipo)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_tipo_modulo IS 'Conjunto de módulos del sistema. Datos estandares de ellos.'; 


CREATE TABLE sis_datos_modulos (
    id_tipo INTEGER,
	nombre VARCHAR (50),
	descripcion VARCHAR (255),
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES sis_sitios (id_objeto), 
    FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo) 
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_datos_modulos IS 'Elemento del conjunto de módulos. Datos generales de el módulo que representa.'; 


CREATE TABLE sis_categorias (
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_modulo) REFERENCES sis_datos_modulos (id_objeto)  
    -- Asociacion de id_contenedor ==> TRIGGER (categoria ó datos_modulo)
) INHERITS (sis_paginas);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_categorias IS 'Clase abstracta para las categorias de objetos de CMS'; 


CREATE TABLE sis_portales (
    PRIMARY KEY (id_objeto)
) INHERITS (sis_sitios);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_portales IS 'Portales del sistema CMS'; 


----------------------------------------------------------------
-- Clases Versiones para los objetos del sistema (Traduccion) --
----------------------------------------------------------------

CREATE TABLE sitio_idioma (
    id_sitio INTEGER,
    id_idioma INTEGER,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) 
  -- Asociacion id_sitio ==> TRIGGER (clase abstracta Sis_sitios)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sitio_idioma IS 'Producto cartesiano entre sitios e idiomas (Relación muchos a ,muchos)'; 


CREATE TABLE sis_versiones (
    id_versiones SERIAL,  
    id_objeto INTEGER NOT NULL,
    id_idioma INTEGER NOT NULL,
    nombre VARCHAR(255),
    descripcion TEXT,
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_objeto) REFERENCES sis_objetos_protegidos (id_objeto),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_versiones IS 'Versiones de los objetos CMS, traducción en los idiomas del sitio.'; 


CREATE TABLE sis_paginas_versiones (
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) ON DELETE CASCADE,
    FOREIGN KEY (id_objeto) REFERENCES sis_paginas (id_objeto) ON DELETE CASCADE
) INHERITS (sis_versiones);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_paginas_versiones IS 'Versiones de los páginas CMS, traducción en los idiomas del sitio.'; 


CREATE TABLE sis_portal_versiones (
    logotipo VARCHAR(100),
    background VARCHAR(100),
    slogan TEXT,
    nombre_pagina VARCHAR(255),
    metatags TEXT,
    metakeywords TEXT,
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_objeto) REFERENCES sis_portales (id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) ON DELETE CASCADE
) INHERITS (sis_versiones); 

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_portal_versiones IS 'Versiones de las cabeceras de un portal, traducción en los idiomas del sitio.'; 


------------------------------------------------------
-- Datos usados para selecciones en los formularios --
------------------------------------------------------

CREATE TABLE sis_actividades_economicas (
    id_actividad SERIAL,
    nombre VARCHAR(255),
    id_idioma INTEGER,
    PRIMARY KEY (id_actividad),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);

CREATE TABLE sis_paises (
    id_paises SERIAL,
    nombre VARCHAR(50),
    id_idioma INTEGER,
    PRIMARY KEY (id_paises),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);


------------------------------------------------------------
-- Asociaciones en clases abtractas o bifurcas (TRIGGERS) --
------------------------------------------------------------

---------- ASOCIACION: sis_roles ---> sis_objetos_protegidos -----------------
-- El id_objeto en sis_roles debe ser un id_objeto en  sis_objetos_protegidos
CREATE OR REPLACE FUNCTION ref_rol_objeto() 
RETURNS TRIGGER AS '
  DECLARE 
	fila sis_objetos_protegidos%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM sis_objetos_protegidos 
	WHERE id_objeto = NEW.id_objeto;

	IF fila.id_objeto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de rol en objeto_protegido'';
        END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_rol 
BEFORE INSERT OR UPDATE ON sis_roles
FOR EACH ROW 
EXECUTE PROCEDURE ref_rol_objeto(sis_roles, id_objeto, sis_objetos_protegidos, id_objeto);

------------- ASOCIACION: sis_usuarios ---> sis_sitios ------------------
-- El id_contenedor en sis_usuarios debe ser un id_objeto en  sis_sitios
CREATE OR REPLACE FUNCTION ref_usuarios_sitios() 
RETURNS TRIGGER AS '
  DECLARE 
	fila sis_sitios%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM sis_sitios
	WHERE id_objeto = NEW.id_contenedor;

	IF fila.id_objeto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de usuario en sitios'';
        END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_usuario_sitio
BEFORE INSERT OR UPDATE ON sis_usuarios
FOR EACH ROW 
EXECUTE PROCEDURE ref_usuarios_sitios(sis_usuarios, id_contenedor, sis_sitios, id_objeto);

------------- ASOCIACION: sis_roles ---> sis_usuarios || sis_grupos ------------------
-- El id_usuario de sis_roles debe estar como id_objeto de sis_usuarios ó id_grupo de sis_grupos
CREATE OR REPLACE FUNCTION ref_roles_usuarios_grupos() 
RETURNS TRIGGER AS '
  DECLARE 
	ident_grupo INTEGER;
	ident_usuario INTEGER;
  BEGIN
	SELECT id_objeto INTO ident_usuario
	FROM sis_usuarios
	WHERE id_objeto = NEW.id_usuario;

	SELECT id_objeto INTO ident_grupo
	FROM sis_grupos
	WHERE id_objeto = NEW.id_usuario;
    
	IF ident_usuario ISNULL AND ident_grupo ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de rol en usuarios o grupos'';
        END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_roles_usuarios_grupos
BEFORE INSERT OR UPDATE ON sis_roles
FOR EACH ROW 
EXECUTE PROCEDURE ref_roles_usuarios_grupos(sis_roles, id_usuario, sis_usuarios, id_objeto);

CREATE TRIGGER referencia_roles_usuarios_grupos2
BEFORE INSERT OR UPDATE ON sis_roles
FOR EACH ROW 
EXECUTE PROCEDURE ref_roles_usuarios_grupos(sis_roles, id_usuario, sis_grupos, id_grupo);


---------------------------------------------------------------------
---------------------------------------------------------------------

CREATE TABLE public.cnt_categorias_contactos
(
  id_categoria serial NOT NULL,
  nombre varchar(255),
  CONSTRAINT cnt_categorias_contactos_pkey PRIMARY KEY (id_categoria)
) WITH OIDS;

CREATE TABLE public.cnt_datos_modulos_contactos
(
  num_forms int4 DEFAULT 10,
  acceso_BD int4 DEFAULT 1,
  num_users int4 DEFAULT 100,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT
) INHERITS (sis_datos_modulos);

CREATE TABLE cnt_formularios
(
  email varchar,
  id_cuestionario int4,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES cnt_datos_modulos_contactos (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
--  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto)
) INHERITS (sis_paginas);

CREATE TABLE cnt_formularios_versiones
(
  text_pre text,
  text_post text,
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES cnt_formularios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) 
) INHERITS (sis_paginas_versiones);

CREATE TABLE cnt_comportamiento_campos
(
  id_formulario int4 NOT NULL,
  apellido1 char(1) DEFAULT 'M',
  apellido2 char(1) DEFAULT 'M',
  tipo_identificacion char(1) DEFAULT 'M',
  email char(1) DEFAULT 'O',
  celular char(1) DEFAULT 'M',
  sexo char(1) DEFAULT 'M',
  nombres char(1) DEFAULT 'M',
  cargo char(1) DEFAULT 'M',
  telefono1 char(1) DEFAULT 'M',
  telefono2 char(1) DEFAULT 'M',
  empresa char(1) DEFAULT 'M',
  identificacion char(1) DEFAULT 'M',
  direccion1 char(1) DEFAULT 'M',
  direccion2 char(1) DEFAULT 'M',
  id_pais char(1) DEFAULT 'M',
  codigo_postal char(1) DEFAULT 'M',
  ciudad char(1) DEFAULT 'M',
  fax char(1) DEFAULT 'M',
  region char(1) DEFAULT 'M',
  PRIMARY KEY (id_formulario),
  FOREIGN KEY (id_formulario) REFERENCES cnt_formularios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) ;


CREATE TABLE public.cnt_contactos_externos
(
  cargo varchar(60),
  direccion1 varchar(100),
  direccion2 varchar(100),
  id_pais int4,
  codigo_postal varchar(10),
  ciudad varchar(100),
  fax varchar(50),
  region varchar(255),
  id_formulario int4,
  id_contenedor int4 NOT NULL,
  nombre_formulario varchar(255),
  id_categoria int4,
  fecha timestamp DEFAULT now(),
  PRIMARY KEY (id_contacto),
  FOREIGN KEY (id_formulario) REFERENCES cnt_formularios (id_objeto) ON UPDATE NO ACTION ON DELETE NO ACTION,
  FOREIGN KEY (id_contenedor) REFERENCES cnt_datos_modulos_contactos (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_categoria) REFERENCES public.cnt_categorias_contactos (id_categoria) ON UPDATE NO ACTION ON DELETE NO ACTION
) INHERITS (sis_contactos);

CREATE TABLE public.cnt_contactos_x_categorias
(
  id_cnt_cat serial NOT NULL,
  id_contacto int4 NOT NULL,
  id_categoria int4 NOT NULL,
  PRIMARY KEY (id_cnt_cat),
  FOREIGN KEY (id_categoria) REFERENCES cnt_categorias_contactos (id_categoria) ON UPDATE CASCADE ON DELETE CASCADE
);
CREATE TABLE public.cnt_orden_campos
(
  id_formulario int4 NOT NULL,
  nombres int4 DEFAULT 1,
  apellido1 int4 DEFAULT 2,
  apellido2 int4 DEFAULT 3,
  tipo_identificacion int4 DEFAULT 4,
  identificacion int4 DEFAULT 5,
  email int4 DEFAULT 6,
  sexo int4 DEFAULT 7,
  empresa int4 DEFAULT 8,
  cargo int4 DEFAULT 9,
  telefono1 int4 DEFAULT 10,
  telefono2 int4 DEFAULT 11,
  celular int4 DEFAULT 12,
  fax int4 DEFAULT 13,
  direccion1 int4 DEFAULT 14,
  direccion2 int4 DEFAULT 15,
  codigo_postal int4 DEFAULT 16,
  id_pais int4 DEFAULT 17,
  region int4 DEFAULT 18,
  ciudad int4 DEFAULT 19,
  observaciones int4 DEFAULT 20,
  PRIMARY KEY (id_formulario),
  FOREIGN KEY (id_formulario) REFERENCES cnt_formularios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE OR REPLACE FUNCTION public.cnt_contactos_x_categorias__sis_contactos()
  RETURNS trigger AS
'
     DECLARE 
	     fila sis_contactos%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM sis_contactos
	     WHERE id_contacto = NEW.id_contacto;
	
	     IF fila.id_contacto ISNULL THEN
	     RAISE EXCEPTION \'Error CMSII: de integridad referencial de cnt_contactos_x_categorias en sis_contactos\';
	     END IF;
		
	     RETURN NEW;
     END;
 '
  LANGUAGE 'plpgsql' VOLATILE;


CREATE TRIGGER cnt_contactos_x_categorias__sis_contactos
BEFORE INSERT OR UPDATE ON cnt_contactos_x_categorias
FOR EACH ROW 
EXECUTE PROCEDURE 
cnt_contactos_x_categorias__sis_contactos('cnt_contactos_x_categorias', 'id_contacto', 'sis_contactos', 'id_contacto');


---------------------------------------------------------------
---------------------------------------------------------------

CREATE TABLE art_datos_modulos_articulos(
  limite_categorias smallint,
  limite_articulos smallint,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES sis_portales (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo)
) INHERITS (sis_datos_modulos);
COMMENT ON TABLE art_datos_modulos_articulos IS 'Datos generales del modulo de contenido.';


CREATE TABLE art_plantillas_categorias(
  id_plantilla serial NOT NULL,
  tipo varchar(100),
  imagen varchar(255),
  PRIMARY KEY (id_plantilla)
);
COMMENT ON  TABLE art_plantillas_categorias IS 'Tipo de plantilla resumen que tiene una categoria en el modulo de contenido';
COMMENT ON  COLUMN art_plantillas_categorias.tipo IS 'tipo de la plantilla, puede ser foto_texto, texto_foto o texto'; 

CREATE TABLE art_categorias_articulos(
  id_plantilla INTEGER DEFAULT 1,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_modulo) REFERENCES art_datos_modulos_articulos (id_objeto),
  FOREIGN KEY (id_plantilla) REFERENCES art_plantillas_categorias (id_plantilla)
) INHERITS (sis_categorias);
COMMENT ON TABLE art_categorias_articulos IS 'Categorias del modulo de contenido';

CREATE TABLE art_categorias_articulos_versiones(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES art_categorias_articulos (id_objeto)  ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones) WITH OIDS;
COMMENT ON TABLE art_categorias_articulos_versiones IS 'Versiones de categorias del modulo de contenido';

CREATE TABLE art_articulos(
  publicado BOOLEAN,
  autor VARCHAR(255),
  fecha_publicacion DATE,
  posicion INTEGER DEFAULT 0,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES art_categorias_articulos (id_objeto)
) INHERITS (sis_paginas);
COMMENT ON TABLE art_articulos IS 'Datos del encabezado del articulo';
COMMENT ON COLUMN art_articulos.posicion IS 'Posicin del articulo dentro de la categoria a la que pertenece'; 

CREATE TABLE art_articulos_versiones(
  imagen varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES art_articulos (id_objeto) ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones);
COMMENT ON TABLE art_articulos_versiones IS 'Versiones del encabezado del articulo';

CREATE TABLE art_bloques(
  posicion INTEGER,
  tipo VARCHAR(50),
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES art_articulos (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  art_bloques IS 'Clase abstracta para los bloques de un articulo';

CREATE TABLE art_bloques_flash(
  flash VARCHAR(255),
  ancho VARCHAR(100),
  alto VARCHAR(100),
  alineacion VARCHAR(100),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON TABLE art_bloques_flash IS 'Bloques de tipo flash';

CREATE TABLE art_bloques_archivos(
  archivo varchar(255),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON  TABLE  art_bloques_archivos IS 'Bloques de tipo archivo';

CREATE TABLE art_bloques_textos(
  texto1 text,
  texto2 text,
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON  TABLE art_bloques_textos IS 'Bloques de tipo texto';

CREATE TABLE art_bloques_texto_fotos(
  texto text,
  foto varchar(255),
  pie_foto varchar(255),
  alt varchar(255),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON  TABLE  art_bloques_texto_fotos IS 'Bloques de tipo texto_foto';


CREATE TABLE public.art_bloques_foto_textos(
  foto varchar(255),
  texto text,
  pie_foto varchar(255),
  alt varchar(255),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON TABLE art_bloques_foto_textos IS 'Bloques de tipo foto_textos';

CREATE TABLE public.art_bloques_imagenes(
  foto1 varchar(255),
  foto2 varchar(255),
  foto3 varchar(255),
  pie_foto1 varchar(255),
  pie_foto2 varchar(255),
  pie_foto3 varchar(255),
  alt1 varchar(255),
  alt2 varchar(255),
  alt3 varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES art_bloques (id_objeto),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (public.sis_versiones);
COMMENT ON TABLE art_bloques_imagenes IS 'Bloques de tipo imagenes';

-- 
-- Trigger para categorias en articulos
-- Author: John A. Vargas, Maria Luisa Manyoma
-- Date: Abril 02 de 2004

--
-- Todo lo que se inserte en art_categoria_articulos
-- El atributo id_contenedor debe estar en art_categoria_articulos
-- O en art_datos_modulos_articulos
--

CREATE  OR REPLACE FUNCTION art_categorias_articulos__art_categorias_articulos() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
		 fila_dat art_datos_modulos_articulos%ROWTYPE;
     BEGIN
		SELECT * INTO fila_dat 
		FROM art_datos_modulos_articulos
		WHERE id_objeto = NEW.id_contenedor;

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_objeto = NEW.id_contenedor;
	
		IF (fila_art.id_objeto ISNULL) AND (fila_dat.id_objeto ISNULL) THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_categorias_articulos__art_categorias_articulos
BEFORE INSERT OR UPDATE ON art_categorias_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_categorias_articulos__art_categorias_articulos('art_categorias_articulos','id_contenedor','art_categorias_articulos','id_objeto');

-- 
-- Cuando borre en art_categorias_articulos
-- verificar que el id_objeto no este como id_contenedor
-- en art_categoria_articulos

CREATE  OR REPLACE FUNCTION art_categorias_articulos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_categorias_articulos__delete
BEFORE DELETE ON art_categorias_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_categorias_articulos__delete();

-- 
-- Cuando borre en art_datos_modulos_articulos
-- verificar que el id_objeto no este como id_contenedor
-- en art_categoria_articulos

CREATE  OR REPLACE FUNCTION art_datos_modulos_articulos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_datos_modulos_articulos__delete
BEFORE DELETE ON art_datos_modulos_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_datos_modulos_articulos__delete();

----------------------------------------------------------------------
----------------------------------------------------------------------

CREATE TABLE gel_datos_modulos_contratos
(
  PRIMARY KEY (id_objeto),
  numero_contratos INTEGER           -- Lmite de contratos
) INHERITS (sis_datos_modulos);


CREATE TABLE gel_dependencias
(
  PRIMARY KEY (id_objeto),
  nombre VARCHAR(255),
  FOREIGN KEY (id_contenedor) REFERENCES gel_datos_modulos_contratos (id_objeto),
  FOREIGN KEY (id_modulo) REFERENCES gel_datos_modulos_contratos (id_objeto)
) INHERITS (sis_objetos_protegidos);

CREATE TABLE gel_contratos_pagina
(
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES gel_datos_modulos_contratos (id_objeto)
) INHERITS (sis_paginas);

CREATE TABLE gel_contratos_pagina_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES gel_contratos_pagina (id_objeto)
) INHERITS (sis_paginas_versiones);


CREATE TABLE gel_estados
(
  id_estado SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);

CREATE TABLE gel_categorias_contratos
(
  id_categoria_contrato SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);

CREATE TABLE gel_tipos_contratos
(
  id_tipo_contrato SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);


CREATE TABLE gel_contratos
(
  id_dependencia INT,
  id_estado INT,
  id_tipo_contrato INT,
  id_categoria_contrato INT,
  numero varchar(255),
  nombre varchar(255),
  objeto text,
  contratista varchar(255),
  fecha_adjudicacion date,
  ordenador_gasto varchar(255),
  activo smallint default '1',
  direccion_compra_pliegos  varchar(255),
  horario_atencion  varchar(255),
  precio integer,
  fecha_hora_apertura_cp timestamp,
  fecha_hora_cierre_cp timestamp,
  departamento varchar(255),
  ciudad varchar(255),
  direccion_propuestas varchar(255),
  fecha_hora_apertura_pr timestamp,
  fecha_hora_cierre_pr timestamp,
  direccion_web_pr varchar(255),
  email_pi varchar(100),
  direccion_web_pi varchar(255), 
  unidad_ejecutora varchar(100),
  presupuesto bigint,    
  PRIMARY KEY (id_objeto), 
  FOREIGN KEY (id_estado) REFERENCES gel_estados(id_estado),
  FOREIGN KEY (id_tipo_contrato) REFERENCES gel_tipos_contratos(id_tipo_contrato),  
  FOREIGN KEY (id_categoria_contrato) REFERENCES gel_categorias_contratos(id_categoria_contrato),    
  FOREIGN KEY (id_modulo) REFERENCES gel_datos_modulos_contratos(id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES gel_datos_modulos_contratos(id_objeto),
  FOREIGN KEY (id_dependencia) REFERENCES gel_dependencias (id_objeto)
) INHERITS (sis_paginas);


CREATE TABLE gel_historial_contratos
(
id_historial SERIAL PRIMARY KEY,
id_contrato INT, 
descripcion VARCHAR(255),
fecha timestamp,
  FOREIGN KEY (id_contrato) REFERENCES gel_contratos(id_objeto)
);

CREATE TABLE gel_comentarios (
id_comentario SERIAL PRIMARY KEY,
id_contrato INT, 
nombre VARCHAR(255),
ciudad VARCHAR(255),
telefono VARCHAR(100),
email VARCHAR(100),
ip inet, 
comentario TEXT,
fecha DATE,
numero VARCHAR(100),
  FOREIGN KEY (id_contrato) REFERENCES gel_contratos(id_objeto)
);

CREATE TABLE gel_tipos_archivos
(
  id_tipo_archivo SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);


CREATE TABLE gel_contratos_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES gel_contratos (id_objeto)
) INHERITS (sis_paginas_versiones);

CREATE TABLE gel_archivos
(
  id_archivo SERIAL PRIMARY KEY,
  id_tipo_archivo smallint,
  nombre VARCHAR(255),
  extension char(3),
  fecha_actualizacion timestamp,
  descargas INT,
  id_versiones INT,
  FOREIGN KEY (id_versiones) REFERENCES gel_contratos_versiones(id_versiones),
  FOREIGN KEY (id_tipo_archivo) REFERENCES gel_tipos_archivos (id_tipo_archivo)  
);

CREATE TABLE gel_proponentes
(
  id_proponente SERIAL PRIMARY KEY,
  id_versiones INT,
  FOREIGN KEY (id_versiones) REFERENCES gel_contratos_versiones(id_versiones),
  nombre VARCHAR(255),
  documento VARCHAR(100),
  representante_legal VARCHAR(255),
  direccion VARCHAR(100),
  telefono VARCHAR(100),
  email VARCHAR(100),
  password VARCHAR(100)
);

---------------------------------------------------------------------
---------------------------------------------------------------------

CREATE TABLE cue_datos_modulos_cuestionarios
(
  num_cues int4 DEFAULT 10,
  num_pregs int4 DEFAULT 30,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo) 
) INHERITS (sis_datos_modulos) ;

CREATE TABLE cue_tipos_preguntas
(
  id_objeto serial NOT NULL,
  id_idioma int4 NOT NULL,
  nombre varchar(255),
  PRIMARY KEY (id_objeto, id_idioma),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) ;

CREATE TABLE cue_cuestionarios
(
  tipo varchar NOT NULL,
  fecha_creacion date DEFAULT now(),
  respuestas bool DEFAULT false,
  PRIMARY KEY (id_objeto)
) INHERITS (sis_paginas);

CREATE TABLE cue_cuestionarios_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (sis_paginas_versiones) ;

CREATE TABLE cue_preguntas
(
  id_pregunta serial NOT NULL,
  id_cuestionario int4,
  tipo varchar,
  numero_preg int2,
  columnas int2,
  filas int2,
  requerida bool DEFAULT 'FALSE',
  PRIMARY KEY (id_pregunta),
  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE cue_preguntas_versiones
(
  texto_post text,
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES cue_preguntas (id_pregunta) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (sis_versiones) ;

CREATE TABLE cue_posibles_respuestas
(
  id_respuesta serial NOT NULL,
  id_pregunta int4,
  tipo varchar,
  numero int2,
  PRIMARY KEY (id_respuesta),
  FOREIGN KEY (id_pregunta) REFERENCES cue_preguntas (id_pregunta) ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE cue_posibles_respuestas_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES cue_posibles_respuestas (id_respuesta) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones) WITH OIDS;

CREATE TABLE cue_respuestas_cuestionarios
(
  id_resp_cue serial NOT NULL,
  id_cuestionario int4,
  fecha timestamp DEFAULT now(),
  PRIMARY KEY (id_resp_cue),
  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE cue_respuestas_preguntas
(
  id_resp_preg serial NOT NULL,
  id_resp_cues int4,
  id_respuesta int4,
  id_pregunta int4,
  texto_resp text,
  PRIMARY KEY (id_resp_preg),
  FOREIGN KEY (id_pregunta) REFERENCES cue_preguntas (id_pregunta) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE cue_rc_anonimos
(
  ip varchar,
  CONSTRAINT cue_rc_anonimos_pkey PRIMARY KEY (id_resp_cue),
  CONSTRAINT "$1" FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (cue_respuestas_cuestionarios) WITH OIDS;

CREATE TABLE public.cue_rc_contactos_nuevos
(
  id_contacto int4,
  PRIMARY KEY (id_resp_cue),
  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
  --FOREIGN KEY (id_contacto) REFERENCES public.cnt_contactos_externos (id_contacto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (cue_respuestas_cuestionarios);

CREATE TABLE cue_rc_registrados
(
  id_usuario int4,
  PRIMARY KEY (id_resp_cue),
  FOREIGN KEY (id_cuestionario) REFERENCES public.cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_usuario) REFERENCES public.sis_usuarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (cue_respuestas_cuestionarios);


------------------------------------------------------------------
-----------------------  TRIGGERS  -------------------------------
------------------------------------------------------------------

CREATE  OR REPLACE FUNCTION cue_preguntas__cue_tipos_preguntas() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila cue_tipos_preguntas%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM cue_tipos_preguntas 
	     WHERE id_objeto = NEW.tipo;
	
	     IF fila.id_objeto ISNULL THEN
	     RAISE EXCEPTION ''Error CMSII: de integridad referencial de cue_preguntas en cue_tipos_preguntas'';
	     END IF;
		
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_preguntas__cue_tipos_preguntas
BEFORE INSERT OR UPDATE ON cue_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_preguntas__cue_tipos_preguntas('cue_preguntas', 'tipo', 'cue_tipos_preguntas', 'id_objeto');

------------------------------------------------------------------

CREATE  OR REPLACE FUNCTION cue_respuestas_preguntas__cue_respuestas_cuestionarios() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila cue_respuestas_cuestionarios%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM cue_respuestas_cuestionarios 
	     WHERE id_resp_cue = NEW.id_resp_cues;
		
	     IF fila.id_resp_cue ISNULL THEN
	     RAISE EXCEPTION ''Error CMSII: de integridad referencial de cue_respuestas_preguntas en cue_respuestas_cuestionarios'';
	     END IF;
		
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_respuestas_preguntas__cue_respuestas_cuestionarios
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cue_respuestas_cuestionarios('cue_respuestas_preguntas', 'id_resp_cues', 'cue_respuestas_cuestionarios', 'id_resp_cue');

------------------------------------------------------------------

CREATE  OR REPLACE FUNCTION cue_respuestas_preguntas__cuestionarios_hijos() 
RETURNS TRIGGER AS '
     BEGIN	
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_respuestas_preguntas__cue_rc_anonimos
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cuestionarios_hijos('cue_respuestas_preguntas', 'id_resp_cues', 'cue_rc_anonimos', 'id_resp_cue');


CREATE TRIGGER cue_respuestas_preguntas__cue_rc_registrados
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cuestionarios_hijos('cue_respuestas_preguntas', 'id_resp_cues', 'cue_rc_registrados', 'id_resp_cue');


CREATE TRIGGER cue_respuestas_preguntas__cue_rc_contactos_nuevos
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cuestionarios_hijos('cue_respuestas_preguntas', 'id_resp_cues', 'cue_rc_contactos_nuevos', 'id_resp_cue');

------------------------------------------------------------------

-- Cuando borre en cue_rc_contactos_nuevos
-- borrar tambien en cue_respuestas_preguntas


CREATE  OR REPLACE FUNCTION cue_rc_contactos_nuevos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_cue cue_respuestas_preguntas%ROWTYPE;
     BEGIN

		DELETE
		FROM cue_respuestas_preguntas
		WHERE id_resp_cues = OLD.id_resp_cue;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_rc_contactos_nuevos__delete
BEFORE DELETE ON cue_rc_contactos_nuevos
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_rc_contactos_nuevos__delete();

------------------------------------------------------------------
-- Cuando borre en cue_rc_registrados
-- borrar tambien en cue_respuestas_preguntas


CREATE  OR REPLACE FUNCTION cue_rc_registrados__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_cue cue_respuestas_preguntas%ROWTYPE;
     BEGIN

		DELETE
		FROM cue_respuestas_preguntas
		WHERE id_resp_cues = OLD.id_resp_cue;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_rc_registrados__delete
BEFORE DELETE ON cue_rc_registrados
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_rc_registrados__delete();

---------------------------------------------------------
---------------------------------------------------------

-- Modulo de directorio
CREATE TABLE dir_datos_modulos_directorio
(
   num_empleados INTEGER,
   PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos);

-- Departamentos (Areas)
CREATE TABLE dir_departamentos 
(
  id_depto SERIAL,
  nombre VARCHAR(255),
  id_modulo INTEGER,
  PRIMARY KEY (id_depto),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio(id_objeto)
);

-- Cargos
CREATE TABLE dir_cargos
(
  id_cargo SERIAL,
  nombre VARCHAR(255),
  manual VARCHAR(255),
  descripcion TEXT,
  id_modulo INTEGER,
  PRIMARY KEY (id_cargo),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio (id_objeto) ON DELETE CASCADE
);

-- Empleados
CREATE TABLE dir_contactos
(
  id_contacto_padre INTEGER,
  id_cargo INTEGER REFERENCES dir_cargos(id_cargo),
  fecha_ingreso DATE,
  fecha_nacimiento DATE,
  foto VARCHAR(255),
  id_modulo INTEGER,
  id_depto INTEGER,
  PRIMARY KEY (id_contacto),
  FOREIGN KEY (id_contacto_padre) REFERENCES dir_contactos (id_contacto),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio (id_objeto) ON DELETE CASCADE,
  FOREIGN KEY (id_cargo) REFERENCES dir_cargos (id_cargo),
  FOREIGN KEY (id_depto) REFERENCES dir_departamentos (id_depto)
  
) INHERITS (sis_contactos);

-- Representacion de la pagina de directorio
CREATE TABLE dir_directorio
(
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio (id_objeto) ON DELETE CASCADE
) INHERITS (sis_paginas);

-- Version en idiomas de la pagina de directorio
CREATE TABLE dir_directorio_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES dir_directorio (id_objeto) ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones);

--
-- TRIGGERS Para llaves foraneas de clases abstractas
--

--
-- Todo lo que se inserte en dir_datos_modulos_directorio
-- El atributo id_contenedor debe estar como id_objeto en sis_sitios
--

CREATE  OR REPLACE FUNCTION modulo_directorio__sis_sitios() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila sis_sitios%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM sis_sitios 
	     WHERE id_objeto = NEW.id_contenedor;
	
	     IF fila.id_objeto ISNULL THEN
	     RAISE EXCEPTION ''Error CMSII: de integridad referencial de datos_modulos directorio en sis_sitios'';
	     END IF;
		
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_modulo_directorio_sis_sitios
BEFORE INSERT OR UPDATE ON dir_datos_modulos_directorio
FOR EACH ROW 
EXECUTE PROCEDURE 
modulo_directorio__sis_sitios('dir_datos_modulos_directorio', 'id_contenedor', 'sis_sitios', 'id_objeto');

----------------------------------------------------------------
----------------------------------------------------------------

CREATE TABLE eve_datos_modulos_eventos (
	limite_categorias smallint,
	PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos) ;
COMMENT ON TABLE eve_datos_modulos_eventos IS 'Tabla que relaciona el modulo de eventos con el sitio';

CREATE TABLE eve_tipo_lugares (
	PRIMARY KEY (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE eve_tipo_lugares IS 'Tabla para diferentes tipos de lugares : gimnasio, auditorio, etc. Es necesario tener al menos uno antes de crear un lugar';

CREATE TABLE eve_tipo_lugares_versiones (
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_tipo_lugares (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones);
COMMENT ON TABLE eve_tipo_lugares_versiones IS 'Parte traducible de los tipos de lugar en cada idioma disponible para el sitio';

CREATE TABLE eve_lugares (
	direccion text,
	tipo integer,
	telefono1 character varying(255),
	telefono2 character varying(255),
	contacto character varying(255),
	email character varying(255),
	pais integer,
	region character varying(255),
	ciudad character varying(255),
	PRIMARY KEY (id_objeto),
	FOREIGN KEY (id_contenedor) REFERENCES eve_datos_modulos_eventos (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE eve_lugares IS 'Lugares donde se realizan los eventos. Es necesario al menos uno antes de crear los eventos';

CREATE TABLE eve_lugares_versiones (
	nombre character varying(255),
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_lugares (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones);
COMMENT ON TABLE eve_lugares_versiones IS 'Parte traducible de los lugares en cada idioma disponible para el sitio';

CREATE TABLE eve_categorias (
	PRIMARY KEY (id_objeto)
) INHERITS (sis_categorias);
COMMENT ON TABLE eve_categorias IS 'Categorias en que son agrupados los eventos';

CREATE TABLE eve_categorias_versiones (
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_categorias (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones);
COMMENT ON TABLE eve_categorias_versiones IS 'Parte traducible de las categorias en cada idioma disponible para el sitio';

CREATE TABLE eve_eventos (
	prioritario character,
	destacado character,
	activo character,
	email_contacto character varying(255),
	lugar integer,
	imagen_chica character varying(255),
	imagen_grande character varying(255),
	valor character varying(255),
	fecha_inicio date,
	fecha_fin date,
	horario character varying(255),
	url character varying(255),
	telefono character varying(255),
	PRIMARY KEY (id_objeto),
	FOREIGN KEY (id_contenedor) REFERENCES eve_categorias (id_objeto),	
	FOREIGN KEY (lugar) REFERENCES eve_lugares (id_objeto)	
) INHERITS (sis_paginas);
COMMENT ON TABLE eve_eventos IS 'Tabla de todos los eventos existentes';

CREATE TABLE eve_eventos_versiones (
	titulo character varying (255),
	descripcion_larga text,
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_eventos (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones);
COMMENT ON TABLE eve_eventos_versiones IS 'Parte traducible de los eventos en cada idioma disponible para el sitio';


  --
-- Todo lo que se inserte en eve_categorias
-- El atributo id_contenedor debe estar en eve_datos_modulos_eventos
--  en eve_categorias
--

CREATE  OR REPLACE FUNCTION ref__eve_categorias()
RETURNS TRIGGER AS '
     DECLARE
             fila eve_datos_modulos_eventos%ROWTYPE;
             fila2 eve_categorias%ROWTYPE;
     BEGIN
                SELECT * INTO fila
                FROM eve_datos_modulos_eventos
                WHERE id_objeto = NEW.id_contenedor;

                SELECT * INTO fila2
                FROM eve_categorias
                WHERE id_objeto = NEW.id_contenedor;

                IF (fila.id_objeto ISNULL) AND (fila2.id_objeto ISNULL) THEN
                   RAISE EXCEPTION ''Error CMSII: de integridad referencial de id_contenedor en eve_categorias'';
                END IF;

                RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_eve_categorias__eve_datos_modulos_categorias
BEFORE INSERT OR UPDATE ON eve_categorias
FOR EACH ROW
EXECUTE PROCEDURE
ref__eve_categorias('eve_categorias','id_contenedor','eve_datos_modulos_eventos','id_objeto');

CREATE TRIGGER ref__eve_categorias__eve_categorias
BEFORE INSERT OR UPDATE ON eve_categorias
FOR EACH ROW
EXECUTE PROCEDURE
ref__eve_categorias('eve_categorias','id_contenedor','eve_categorias','id_objeto');

--
-- Borrado
--

CREATE OR REPLACE FUNCTION eve_categorias__delete()
  RETURNS trigger AS
'
     DECLARE
             fila eve_categorias%ROWTYPE;
     BEGIN

                SELECT * INTO fila
                FROM eve_categorias
                WHERE id_contenedor = OLD.id_objeto;

                IF fila.id_objeto THEN
                   RAISE EXCEPTION \'Error de integridad referencial de contenedor en categoria de eventos\';
                        END IF;

                RETURN OLD;
     END;
 '
  LANGUAGE 'plpgsql';

CREATE TRIGGER ref_eve_categorias__eve_categorias_delete
BEFORE DELETE ON eve_categorias
FOR EACH ROW
EXECUTE PROCEDURE
eve_categorias__delete();

----------------------------------------------------------------
----------------------------------------------------------------

CREATE TABLE fot_datos_modulos_foto_album (
	limite_categorias smallint,
	limite_colecciones smallint,
	limite_imagenes smallint,
	PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos) ;
COMMENT ON TABLE fot_datos_modulos_foto_album IS 'Tabla que relaciona el modulo con un sitio. Contiene los limites de categorias, colecciones e imagenes que se pueden almacenar.';

CREATE TABLE fot_categorias (
    img_archivo character varying(255),
    img_pie character varying(255),
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_datos_modulos_foto_album(id_objeto)
)INHERITS (sis_categorias);
COMMENT ON TABLE fot_categorias IS 'Categorias de imagenes del sitio';

CREATE TABLE fot_categorias_versiones (
	nombre character varying(255),
	descripcion text,
    PRIMARY KEY (id_objeto, id_idioma),
    FOREIGN KEY (id_objeto) REFERENCES fot_categorias(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_paginas_versiones);
COMMENT ON TABLE fot_categorias_versiones IS 'Parte traducible de las categorias para cada idioma disponible en el sitio';

CREATE TABLE fot_colecciones (
    -- id_contenedor es el id_objeto de la categoria a la que pertenece
    img_archivo character varying(255),
	posicion integer NOT NULL,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_categorias(id_objeto)
)INHERITS (sis_paginas);
COMMENT ON TABLE fot_colecciones IS 'Colecciones en las que estan divididas las categorias';

CREATE TABLE fot_colecciones_versiones (
	nombre character varying(255),
	descripcion text,
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_objeto) REFERENCES fot_colecciones(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_paginas_versiones);
COMMENT ON TABLE fot_colecciones_versiones IS 'Parte traducible de las colecciones para cada idioma disponible en el sitio';

CREATE TABLE fot_imagenes (
    img_archivo character varying(255),
    img_archivo_thumb character varying(255),
    posicion integer,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_colecciones(id_objeto)
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE fot_imagenes IS 'Todas las imagenes del foto album';

CREATE TABLE fot_imagenes_versiones (
	descripcion text,
    PRIMARY KEY (id_objeto, id_idioma),
    FOREIGN KEY (id_objeto) REFERENCES fot_imagenes(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_paginas_versiones);
COMMENT ON TABLE fot_imagenes_versiones IS 'Parte traducible de las imagenes para cada idioma disponible en el sitio';

CREATE TABLE fot_atributos (
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_categorias(id_objeto) ON DELETE CASCADE
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE fot_atributos IS 'Atributos que se le pueden asignar a cada categoria, para que sus valores sean diligenciados en cada coleccion perteneciente a esta';

CREATE TABLE fot_atributos_versiones (
    PRIMARY KEY (id_objeto, id_idioma),
    FOREIGN KEY (id_objeto) REFERENCES fot_atributos(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_versiones);
COMMENT ON TABLE fot_atributos_versiones IS 'Parte traducible de los atributos por cada idioma disponible en el sitio';

CREATE TABLE fot_atributos_valores (
    id_coleccion INTEGER,
    valor TEXT,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_atributos(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_coleccion) REFERENCES fot_colecciones_versiones(id_versiones) ON DELETE CASCADE
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE fot_atributos_valores IS 'Valores de los atributos de la categoria por cada coleccion. Los valores son asignados a la version de la coleccion';

---------------------------------------------------------------------
---------------------------------------------------------------------

CREATE TABLE men_items

(

  CONSTRAINT men_items_pkey PRIMARY KEY (id_objeto)

) INHERITS (sis_objetos_protegidos);



CREATE TABLE men_items_versiones

(

  imagen_over VARCHAR,

  imagen_up VARCHAR,

  PRIMARY KEY (id_objeto, id_idioma),

  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) ON DELETE CASCADE

) INHERITS (sis_versiones);



CREATE TABLE men_datos_modulos_menus

(

  nombre VARCHAR(50),

  descripcion VARCHAR(255),

  id_tipo INTEGER,

  PRIMARY KEY (id_objeto),

  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo)

) INHERITS (sis_datos_modulos);



CREATE TABLE men_menus

(

  id_index int4,

  tipo_index CHAR(1) DEFAULT 'i',

  tipo CHAR(1) DEFAULT 'v',

  CONSTRAINT men_menus_pkey PRIMARY KEY (id_objeto)

) INHERITS (men_items);



CREATE TABLE men_enlaces

(

  CONSTRAINT men_enlaces_pkey PRIMARY KEY (id_objeto)

) INHERITS (men_items) ;



CREATE TABLE men_enlaces_externos

(

  url VARCHAR,

  PRIMARY KEY (id_objeto)

) INHERITS (men_enlaces) ;



CREATE TABLE men_enlaces_internos

(

  id_pagina INTEGER,

  PRIMARY KEY (id_objeto)

) INHERITS (men_enlaces) ;


------------------------------------------------------------------
-----------------------  TRIGGERS  -------------------------------
------------------------------------------------------------------

CREATE OR REPLACE FUNCTION men_items_versiones__men_items()

  RETURNS trigger AS
'
     DECLARE 
	     fila men_items%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM men_items 
	     WHERE id_objeto = NEW.id_objeto;
	
	     IF fila.id_objeto ISNULL THEN
	     RAISE EXCEPTION \'Error CMSII: de integridad referencial de men_items_versiones en men_items\';
	     END IF;
		
	     RETURN NEW;
     END;
 '
  LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER men_items_versiones__men_items

BEFORE INSERT OR UPDATE ON men_items_versiones

FOR EACH ROW 

EXECUTE PROCEDURE 

men_items_versiones__men_items('men_items_versiones', 'id_objeto', 'men_items', 'id_objeto');


------------------------------------------------------------------
----------------------------------------------------------------

CREATE TABLE prd_datos_modulos_productos
(
  limite_categorias SMALLINT,
  limite_productos SMALLINT,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo)
) INHERITS (sis_datos_modulos);
COMMENT ON TABLE prd_datos_modulos_productos IS 'Datos generales del modulo del productos del catalogo.';

CREATE TABLE prd_categorias_productos
(
  PRIMARY KEY (id_objeto)
) INHERITS (sis_categorias);
COMMENT ON  TABLE prd_categorias_productos IS 'Categorias de productos del catalogo.';

CREATE TABLE prd_categorias_productos_versiones
(
  imagen varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_categorias_productos (id_objeto)
) INHERITS (sis_paginas_versiones);
COMMENT ON  TABLE  prd_categorias_productos_versiones IS 'Versiones(idiomas) de cada una de las categorias de productos del catalogo.';

CREATE TABLE prd_productos
(
  publicado BOOLEAN,
  precio FLOAT,
  posicion INTEGER,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES prd_categorias_productos (id_objeto)
) INHERITS (sis_paginas);
COMMENT ON  TABLE  prd_productos IS 'Productos del catalogo.';


CREATE TABLE prd_productos_versiones
(
  imagen varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_productos (id_objeto) ON DELETE CASCADE
) INHERITS (sis_paginas_versiones);
COMMENT ON  TABLE  prd_productos IS 'Versiones(idiomas) de cada uno de los productos del catalogo.';

CREATE TABLE prd_atributos
(
  protegido BOOLEAN DEFAULT false,
  tipo varchar(255),
  caracteristica varchar(100),
  PRIMARY KEY (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  prd_atributos IS 'Nombres de cada uno de los atributos de un  producto o una categoria del catalogo.';
COMMENT ON COLUMN prd_atributos.tipo IS 'Indica si el atributo pertenece a una categoria o un producto'; 
COMMENT ON COLUMN prd_atributos.caracteristica IS 'Indica si el atributo es lista de selección o texto'; 

CREATE TABLE prd_atributos_valores
(
  protegido BOOLEAN DEFAULT false,
  valor text,
  id_producto int4,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES prd_atributos (id_objeto),
  FOREIGN KEY (id_producto) REFERENCES prd_productos_versiones (id_versiones) ON DELETE CASCADE
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  prd_atributos_valores IS 'Valores que toman cada uno de los atributos.';

CREATE TABLE prd_atributos_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_atributos (id_objeto) ON DELETE CASCADE
) INHERITS (sis_versiones);
COMMENT ON  TABLE  prd_atributos IS 'Versiones de cada atributo.';

CREATE TABLE prd_atributos_opciones
(
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES prd_atributos (id_objeto) 
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  prd_atributos IS 'Opciones que tienen los atributos de tipo lista .';


CREATE TABLE prd_atributos_opciones_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_atributos_opciones (id_objeto) ON DELETE CASCADE
) INHERITS (public.sis_versiones) WITH OIDS;
COMMENT ON  TABLE  prd_atributos IS 'Versiones de las opciones que toman los atributos de tipo lista .';



-- 
-- Trigger para categorias en productos
-- Author: John A. Vargas, Maria Luisa Manyoma
-- Date: Abril 02 de 2004

--
-- Todo lo que se inserte en prd_categoria_productos
-- El atributo id_contenedor debe estar en prd_categoria_productos
-- O en prd_datos_modulos_productos
--

CREATE  OR REPLACE FUNCTION prd_categorias_productos__prd_categorias_productos() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
		 fila_dat prd_datos_modulos_productos%ROWTYPE;
     BEGIN
		SELECT * INTO fila_dat 
		FROM prd_datos_modulos_productos
		WHERE id_objeto = NEW.id_contenedor;

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_objeto = NEW.id_contenedor;
	
		IF (fila_art.id_objeto ISNULL) AND (fila_dat.id_objeto ISNULL) THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria Producto'';
			END IF;
		
		RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_categorias_productos__prd_categorias_productos 
BEFORE INSERT OR UPDATE ON prd_categorias_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_categorias_productos__prd_categorias_productos('prd_categorias_productos','id_contenedor','prd_categorias_productos','id_objeto');

-- 
-- Cuando borre en prd_categorias_productos
-- verificar que el id_objeto no este como id_contenedor
-- en prd_categoria_productos

CREATE  OR REPLACE FUNCTION prd_categorias_productos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_categorias_productos__delete
BEFORE DELETE ON prd_categorias_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_categorias_productos__delete();

-- 
-- Cuando borre en prd_datos_modulos_productos
-- verificar que el id_objeto no este como id_contenedor
-- en prd_categoria_productos

CREATE  OR REPLACE FUNCTION prd_datos_modulos_productos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_datos_modulos_productos__delete
BEFORE DELETE ON prd_datos_modulos_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_datos_modulos_productos__delete();
