<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
/* +----------------------------------------------------------------------+
 * | PHP version 4.1.2                                                    |
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2003 El grupo Emedia - Telesat                         |
 * +----------------------------------------------------------------------+
 * | Authors: Maria Luisa Manyoma <mlmanyoma@telesat.com.co>              |
 * |          John Vargas <jhonvargas@telesat.com.co>                     |
 * |          Camilo Torres <ccordoba@telesat.com.co>                     |
 * |          Junior Tello <rtello@telesat.com.co>                        |
 * +----------------------------------------------------------------------+
 * 
 */
 /**
 * Especializacion de DataObject para el uso de objeto-relacion de PostgreSQL
 * http://pear.php.net/manual/en/package.database.db-dataobject.php
 *
 * @package DB_DataObject
 * @version  1
 * @author John A. Vargas
 * @since    PHP 4.1.2
 */
require_once 'DB/DataObject.php';

Class CMS_DataObject extends DB_DataObject
{
    /**
     * Inserta un objeto en la base de datos esta clase hereda de la 
	 * clase DB_DataObject de PEAR y hace la insercin usando la secuencia
	 * de a clase padre mas ancestra en el arbol de herencia. 
     *
     * 
     * @access public
     * @return  mixed|false key value or false on failure
	 * @author John A. Vargas
     */
	 function insert()
	 {  
	 	global $_DB_DATAOBJECT;
        // connect will load the config!
        $this->_connect();
		
        $keys      = $this->keys();
		$__DB  = &$_DB_DATAOBJECT['CONNECTIONS'][$this->_database_dsn_md5];
  
        // big check for using sequences
        if ($key = @$keys[0])
			if (!isset($this->$key))
	        {
				$my_options = &PEAR::getStaticProperty('DB_DataObject','options');
				$class_prefix  = $my_options['class_prefix'];
				
				$obj = $this;
				while (get_parent_class($obj) != "cms_dataobject") {
					$class_padre = $class_prefix.ucfirst(get_parent_class($obj));
					$obj = new $class_padre;
				}	
				$seq = get_class($obj) . '_' . $key ;
			
				$this->$key = $__DB->nextId($seq);
			}
		
		$_DB_DATAOBJECT['CONFIG']['ignore_sequence_keys'] = 'ALL';
		
		// Se llama al metodo de la clase padre
		parent::insert();	

		return $this->$key;
	 }


    /**
     * Esta implementacion de UPDATE soluciona el bug para datos de tipo
	 *  boolean en PostgreSQL
	 *
     * @param object dataobject (optional) - used to only update changed items.
     * @access public
     * @return  int rows affected or false on failure
	 * @author John A. Vargas
     */
    function update($dataObject = false)
    {
        $items = $this->table(); 

        foreach($items as $k => $v) {
            if (!isset($this->$k)) {
                continue;
            }
            if (($dataObject !== false) && (@$dataObject->$k == $this->$k)) {
                continue;
            }

            if ($v & DB_DATAOBJECT_BOOL) {
				if ($this->$k == 't') {
					 $this->$k = "TRUE";
					continue;
				} elseif ($this->$k == 'f') {
					 $this->$k = "FALSE";
					continue;
				}
            }
        }
		
		// Se llama al metodo de la clase padre
		return parent::update($dataObject);
		
    }
	
    /**
	 * Este mtodo busca el objeto de la clase mas especializada
	 * de el objeto, lo hace recursivamente bajando en el rbol de
	 * herencia hasta llegar a una hoja de este.
	 * 
	 * Ej: 
	 * // si B ---> A,  
	 * $b = new A;  $b->get($id);
	 * $b = $b->get_hijo();  // $b sera una instancia de B
     *
     * @access public
	 * @return Object Objeto de la clase (Y tabla) hijo en el arbol de herencia
	 * @author John A. Vargas
     */
	function get_hijo()
	{
		$nombre_clase = get_class($this);
		$clases = get_declared_classes(); // Clases declaradas en el sistema
	
		foreach ($clases as $clase) {
			// Tomar las clases hijas de la clase del objeto
			if (get_parent_class($clase) == $nombre_clase) {
				$obj_hijo = new $clase;
				$llaves = $this->keys(); // Obtenemos las llaves de la clase en la DB
				foreach ($llaves as $llave)
					$obj_hijo->$llave = $this->$llave;

				if ($obj_hijo->find(true)) {		
					return $obj_hijo->get_hijo ();
				}
			}
		}	
		
		return $this;
	}

    /**
     * Imprime la consulta SQL del DataObject para
	 * depurar errores.
     *
     * @param   boolean $n Fetch first result
     * @access  public
     * @return  int
     */
    function print_query()
    {
        echo "<br>QUERY:";
	echo 'SELECT ' .
            $this->_query['data_select'] .
            ' FROM ' . ($quoteIdentifiers ? $DB->quoteIdentifier($this->__table) : $this->__table) . " " .
            $this->_join .
            $this->_query['condition'] . ' '.
            $this->_query['group_by']  . ' '.
            $this->_query['having']    . ' '.
            $this->_query['order_by']  . ' '.
            
            $this->_query['limit']. '<br>';
    }
}

?>