----------------------------------------------------------------

-- Archivo de creacin de tablas para el paquete de Cines     --

-- Fecha: Septiembre 14 de 2004                               --

-- Autor: John A. Vargas <jhonvargas@telesat.com.co>          --

----------------------------------------------------------------



-- Tabla agrupadora de los datos del mdulo

CREATE TABLE cin_datos_modulos_cines (

	id_articulo INTEGER,

    PRIMARY KEY (id_objeto)

) INHERITS (sis_datos_modulos);



COMMENT ON TABLE cin_datos_modulos_cines IS 'Agrupa los datos del mdulo de cines';  

COMMENT ON COLUMN cin_datos_modulos_cines.id_articulo IS 'Identificador del articulo con la pelicula recomendada';  



-- Tabla que representa la pgina enlazable del men para cines

CREATE TABLE cin_paginas (

    PRIMARY KEY (id_objeto),

    FOREIGN KEY (id_contenedor) REFERENCES cin_datos_modulos_cines (id_objeto)

) INHERITS (sis_paginas);



-- Versin en idiomas de la pgina  para cines

CREATE TABLE cin_paginas_versiones (

    PRIMARY KEY (id_versiones),

    FOREIGN KEY (id_objeto) REFERENCES cin_paginas (id_objeto),

    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)

) INHERITS (sis_paginas_versiones);



-- Los teatros del cine

CREATE TABLE cin_teatros (

    id_teatro SERIAL,

    nombre VARCHAR(255),

    telefono VARCHAR(255),

    direccion VARCHAR(255),

    tarifa VARCHAR(255),

    zona VARCHAR(255),

    ciudad VARCHAR(255),

    id_contenedor INTEGER,

    reserva INTEGER,

    PRIMARY KEY (id_teatro),

    FOREIGN KEY (id_contenedor) REFERENCES cin_datos_modulos_cines (id_objeto)

);



-- Clasificados de peliculas

CREATE TABLE cin_clasificaciones (

    id_clasificacion SERIAL,

    nombre VARCHAR(255),

    PRIMARY KEY (id_clasificacion)

);



-- Calificacion de peliculas

CREATE TABLE cin_calificaciones (

    id_calificacion SERIAL,

    nombre VARCHAR(255),

    PRIMARY KEY (id_calificacion)

);



-- Clasificados de peliculas

CREATE TABLE cin_generos (

    id_genero SERIAL,

    nombre VARCHAR(255),

    PRIMARY KEY (id_genero)

);



-- Las peliculas vistas en el mdulo de cine

CREATE TABLE cin_peliculas (

    id_pelicula SERIAL,

    titulo_espanol VARCHAR(255),

    titulo_original VARCHAR(255),

    tag_line VARCHAR(255),

    id_clasificacion INTEGER,

    id_calificacion INTEGER,

    id_genero INTEGER,

    reparto TEXT,

    director VARCHAR(255),

    sinopsis TEXT,

    alquilar INTEGER,

    estreno INTEGER,

    destacado INTEGER,

    imagen VARCHAR(255),

    id_contenedor INTEGER,

	pagina_oficial VARCHAR(255),

	id_encuesta INTEGER,

    PRIMARY KEY (id_pelicula),

    FOREIGN KEY (id_clasificacion) REFERENCES cin_clasificaciones (id_clasificacion),

    FOREIGN KEY (id_calificacion) REFERENCES cin_calificaciones (id_calificacion),

    FOREIGN KEY (id_genero) REFERENCES cin_generos (id_genero),

    FOREIGN KEY (id_contenedor) REFERENCES cin_datos_modulos_cines (id_objeto)

);



COMMENT ON TABLE cin_peliculas IS 'Peliculas a presentar en el mdulo de cines';  



-- Funciones de las peliculas en los teatros

CREATE TABLE cin_funciones (

    id_funcion SERIAL,

    id_teatro INTEGER,

    id_pelicula INTEGER,

    horario TEXT,

    desde DATE,

    hasta DATE,

    PRIMARY KEY (id_funcion),

    FOREIGN KEY (id_teatro) REFERENCES cin_teatros (id_teatro),

    FOREIGN KEY (id_pelicula) REFERENCES cin_peliculas (id_pelicula)

);



COMMENT ON TABLE cin_funciones IS 'Funciones de las peliculas en los teatros';  

