---------------------------------------------------------------------
-- Autor: JUNIOR TELLO, juniorts7@yahoo.com, rtello@telesat.com.co --
-- Archivo de creacin de tablas para el paquete contactos         --
-- Fecha: 14 de Abril de 2004                                      --
---------------------------------------------------------------------

-- Posterior a create_sistema.sql
-- Posterior a create.sql del paquete cuestionarios

CREATE TABLE public.cnt_categorias_contactos
(
  id_categoria serial NOT NULL,
  nombre varchar(255),
  CONSTRAINT cnt_categorias_contactos_pkey PRIMARY KEY (id_categoria)
) WITH OIDS;

CREATE TABLE public.cnt_datos_modulos_contactos
(
  num_forms int4 DEFAULT 10,
  acceso_BD int4 DEFAULT 1,
  num_users int4 DEFAULT 100,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo) ON UPDATE RESTRICT ON DELETE RESTRICT
) INHERITS (sis_datos_modulos);

CREATE TABLE cnt_formularios
(
  email varchar,
  id_cuestionario int4,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES cnt_datos_modulos_contactos (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
--  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto)
) INHERITS (sis_paginas);

CREATE TABLE cnt_formularios_versiones
(
  text_pre text,
  text_post text,
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES cnt_formularios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) 
) INHERITS (sis_paginas_versiones);

CREATE TABLE cnt_comportamiento_campos
(
  id_formulario int4 NOT NULL,
  apellido1 char(1) DEFAULT 'M',
  apellido2 char(1) DEFAULT 'M',
  tipo_identificacion char(1) DEFAULT 'M',
  email char(1) DEFAULT 'O',
  celular char(1) DEFAULT 'M',
  sexo char(1) DEFAULT 'M',
  nombres char(1) DEFAULT 'M',
  cargo char(1) DEFAULT 'M',
  telefono1 char(1) DEFAULT 'M',
  telefono2 char(1) DEFAULT 'M',
  empresa char(1) DEFAULT 'M',
  identificacion char(1) DEFAULT 'M',
  direccion1 char(1) DEFAULT 'M',
  direccion2 char(1) DEFAULT 'M',
  id_pais char(1) DEFAULT 'M',
  codigo_postal char(1) DEFAULT 'M',
  ciudad char(1) DEFAULT 'M',
  fax char(1) DEFAULT 'M',
  region char(1) DEFAULT 'M',
  PRIMARY KEY (id_formulario),
  FOREIGN KEY (id_formulario) REFERENCES cnt_formularios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE public.cnt_contactos_externos
(
  cargo varchar(60),
  direccion1 varchar(100),
  direccion2 varchar(100),
  id_pais int4,
  codigo_postal varchar(10),
  ciudad varchar(100),
  fax varchar(50),
  region varchar(255),
  id_formulario int4,
  id_contenedor int4 NOT NULL,
  nombre_formulario varchar(255),
  id_categoria int4,
  fecha timestamp DEFAULT now(),
  PRIMARY KEY (id_contacto),
  FOREIGN KEY (id_formulario) REFERENCES cnt_formularios (id_objeto) ON UPDATE NO ACTION ON DELETE NO ACTION,
  FOREIGN KEY (id_contenedor) REFERENCES cnt_datos_modulos_contactos (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_categoria) REFERENCES cnt_categorias_contactos (id_categoria) ON UPDATE NO ACTION ON DELETE NO ACTION
) INHERITS (sis_contactos);

CREATE TABLE public.cnt_contactos_x_categorias
(
  id_cnt_cat serial NOT NULL,
  id_contacto int4 NOT NULL,
  id_categoria int4 NOT NULL,
  PRIMARY KEY (id_cnt_cat),
  FOREIGN KEY (id_categoria) REFERENCES cnt_categorias_contactos (id_categoria) ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE public.cnt_orden_campos
(
  id_formulario int4 NOT NULL,
  nombres int4 DEFAULT 1,
  apellido1 int4 DEFAULT 2,
  apellido2 int4 DEFAULT 3,
  tipo_identificacion int4 DEFAULT 4,
  identificacion int4 DEFAULT 5,
  email int4 DEFAULT 6,
  sexo int4 DEFAULT 7,
  empresa int4 DEFAULT 8,
  cargo int4 DEFAULT 9,
  telefono1 int4 DEFAULT 10,
  telefono2 int4 DEFAULT 11,
  celular int4 DEFAULT 12,
  fax int4 DEFAULT 13,
  direccion1 int4 DEFAULT 14,
  direccion2 int4 DEFAULT 15,
  codigo_postal int4 DEFAULT 16,
  id_pais int4 DEFAULT 17,
  region int4 DEFAULT 18,
  ciudad int4 DEFAULT 19,
  observaciones int4 DEFAULT 20,
  PRIMARY KEY (id_formulario),
  FOREIGN KEY (id_formulario) REFERENCES cnt_formularios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
);

------------------------------------------------------------------
-----------------------  TRIGGERS  -------------------------------
------------------------------------------------------------------
-- Al insertar o actualizar en cnt_contactos_x_categorias chequear que el 
-- id_contacto de sis_contactos est en la vista.
CREATE OR REPLACE FUNCTION public.cnt_contactos_x_categorias__sis_contactos()
  RETURNS trigger AS
'
     DECLARE 
	     fila sis_contactos%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM sis_contactos
	     WHERE id_contacto = NEW.id_contacto;
	
	     IF fila.id_contacto ISNULL THEN
	     RAISE EXCEPTION \'Error CMSII: de integridad referencial de cnt_contactos_x_categorias en sis_contactos\';
	     END IF;
		
	     RETURN NEW;
     END;
 '
  LANGUAGE 'plpgsql' VOLATILE;


CREATE TRIGGER cnt_contactos_x_categorias__sis_contactos
BEFORE INSERT OR UPDATE ON cnt_contactos_x_categorias
FOR EACH ROW 
EXECUTE PROCEDURE 
cnt_contactos_x_categorias__sis_contactos('cnt_contactos_x_categorias', 'id_contacto', 'sis_contactos', 'id_contacto');
--  THE END  --