-- Archivo de creacin de tablas para el paquete Articulos
-- Fecha: Mayo 14 2004
-- Autor Maria Luisa Manyoma

CREATE TABLE art_datos_modulos_articulos(
  limite_categorias smallint,
  limite_articulos smallint,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES sis_portales (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo)
) INHERITS (sis_datos_modulos);
COMMENT ON TABLE art_datos_modulos_articulos IS 'Datos generales del modulo de contenido.';


CREATE TABLE art_plantillas_categorias(
  id_plantilla serial NOT NULL,
  tipo varchar(100),
  imagen varchar(255),
  PRIMARY KEY (id_plantilla)
);
COMMENT ON  TABLE art_plantillas_categorias IS 'Tipo de plantilla resumen que tiene una categoria en el modulo de contenido';
COMMENT ON  COLUMN art_plantillas_categorias.tipo IS 'tipo de la plantilla, puede ser foto_texto, texto_foto o texto'; 

CREATE TABLE art_categorias_articulos(
  id_plantilla INTEGER DEFAULT 1,
  id_menu INTEGER,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_modulo) REFERENCES art_datos_modulos_articulos (id_objeto),
  FOREIGN KEY (id_plantilla) REFERENCES art_plantillas_categorias (id_plantilla),
  FOREIGN KEY (id_menu) REFERENCES Men_menus (id_objeto)
) INHERITS (sis_categorias);
COMMENT ON TABLE art_categorias_articulos IS 'Categorias del modulo de contenido';

CREATE TABLE art_categorias_articulos_versiones(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES art_categorias_articulos (id_objeto)  ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones) WITH OIDS;
COMMENT ON TABLE art_categorias_articulos_versiones IS 'Versiones de categorias del modulo de contenido';

CREATE TABLE art_articulos(
  publicado BOOLEAN,
  autor VARCHAR(255),
  fecha_publicacion DATE,
  posicion INTEGER DEFAULT 0,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES art_categorias_articulos (id_objeto)
) INHERITS (sis_paginas);
COMMENT ON TABLE art_articulos IS 'Datos del encabezado del articulo';
COMMENT ON COLUMN art_articulos.posicion IS 'Posicin del articulo dentro de la categoria a la que pertenece'; 

CREATE TABLE art_articulos_versiones(
  imagen varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES art_articulos (id_objeto) ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones);
COMMENT ON TABLE art_articulos_versiones IS 'Versiones del encabezado del articulo';

CREATE TABLE art_bloques(
  posicion INTEGER,
  tipo VARCHAR(50),
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES art_articulos (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  art_bloques IS 'Clase abstracta para los bloques de un articulo';

CREATE TABLE art_bloques_flash(
  flash VARCHAR(255),
  ancho VARCHAR(100),
  alto VARCHAR(100),
  alineacion VARCHAR(100),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON TABLE art_bloques_flash IS 'Bloques de tipo flash';

CREATE TABLE art_bloques_archivos(
  archivo varchar(255),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON  TABLE  art_bloques_archivos IS 'Bloques de tipo archivo';

CREATE TABLE art_bloques_textos(
  texto1 text,
  texto2 text,
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON  TABLE art_bloques_textos IS 'Bloques de tipo texto';

CREATE TABLE art_bloques_texto_fotos(
  texto text,
  foto varchar(255),
  pie_foto varchar(255),
  alt varchar(255),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON  TABLE  art_bloques_texto_fotos IS 'Bloques de tipo texto_foto';


CREATE TABLE public.art_bloques_foto_textos(
  foto varchar(255),
  texto text,
  pie_foto varchar(255),
  alt varchar(255),
  PRIMARY KEY (id_versiones)
) INHERITS (sis_versiones);
COMMENT ON TABLE art_bloques_foto_textos IS 'Bloques de tipo foto_textos';

CREATE TABLE public.art_bloques_imagenes(
  foto1 varchar(255),
  foto2 varchar(255),
  foto3 varchar(255),
  pie_foto1 varchar(255),
  pie_foto2 varchar(255),
  pie_foto3 varchar(255),
  alt1 varchar(255),
  alt2 varchar(255),
  alt3 varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES art_bloques (id_objeto),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (public.sis_versiones);
COMMENT ON TABLE art_bloques_imagenes IS 'Bloques de tipo imagenes';

-- 
-- Trigger para categorias en articulos
-- Author: John A. Vargas, Maria Luisa Manyoma
-- Date: Abril 02 de 2004

--
-- Todo lo que se inserte en art_categoria_articulos
-- El atributo id_contenedor debe estar en art_categoria_articulos
-- O en art_datos_modulos_articulos
--

CREATE  OR REPLACE FUNCTION art_categorias_articulos__art_categorias_articulos() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
		 fila_dat art_datos_modulos_articulos%ROWTYPE;
     BEGIN
		SELECT * INTO fila_dat 
		FROM art_datos_modulos_articulos
		WHERE id_objeto = NEW.id_contenedor;

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_objeto = NEW.id_contenedor;
	
		IF (fila_art.id_objeto ISNULL) AND (fila_dat.id_objeto ISNULL) THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_categorias_articulos__art_categorias_articulos
BEFORE INSERT OR UPDATE ON art_categorias_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_categorias_articulos__art_categorias_articulos('art_categorias_articulos','id_contenedor','art_categorias_articulos','id_objeto');

-- 
-- Cuando borre en art_categorias_articulos
-- verificar que el id_objeto no este como id_contenedor
-- en art_categoria_articulos

CREATE  OR REPLACE FUNCTION art_categorias_articulos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_categorias_articulos__delete
BEFORE DELETE ON art_categorias_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_categorias_articulos__delete();

-- 
-- Cuando borre en art_datos_modulos_articulos
-- verificar que el id_objeto no este como id_contenedor
-- en art_categoria_articulos

CREATE  OR REPLACE FUNCTION art_datos_modulos_articulos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_datos_modulos_articulos__delete
BEFORE DELETE ON art_datos_modulos_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_datos_modulos_articulos__delete();