-- 
-- Trigger para categorias en articulos
-- Author: John A. Vargas, Maria Luisa Manyoma
-- Date: Abril 02 de 2004

--
-- Todo lo que se inserte en art_categoria_articulos
-- El atributo id_contenedor debe estar en art_categoria_articulos
-- O en art_datos_modulos_articulos
--

CREATE  OR REPLACE FUNCTION art_categorias_articulos__art_categorias_articulos() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
		 fila_dat art_datos_modulos_articulos%ROWTYPE;
     BEGIN
		SELECT * INTO fila_dat 
		FROM art_datos_modulos_articulos
		WHERE id_objeto = NEW.id_contenedor;

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_objeto = NEW.id_contenedor;
	
		IF (fila_art.id_objeto ISNULL) AND (fila_dat.id_objeto ISNULL) THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_categorias_articulos__art_categorias_articulos
BEFORE INSERT OR UPDATE ON art_categoria_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_categorias_articulos__art_categorias_articulos('art_categorias_articulos','id_contenedor','art_categorias_articulos','id_objeto');

-- 
-- Cuando borre en art_categorias_articulos
-- verificar que el id_objeto no este como id_contenedor
-- en art_categoria_articulos

CREATE  OR REPLACE FUNCTION art_categorias_articulos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_categorias_articulos__delete
BEFORE DELETE ON art_categoria_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_categorias_articulos__delete();

-- 
-- Cuando borre en art_datos_modulos_articulos
-- verificar que el id_objeto no este como id_contenedor
-- en art_categoria_articulos

CREATE  OR REPLACE FUNCTION art_datos_modulos_articulos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art art_categorias_articulos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM art_categorias_articulos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_art_datos_modulos_articulos__delete
BEFORE DELETE ON art_datos_modulos_articulos
FOR EACH ROW 
EXECUTE PROCEDURE 
art_datos_modulos_articulos__delete();