----------------------------------------------------------------------
-- Archivo de creacin de tablas para el paquete Gobierno En Lnea  --
-- Fecha: Agosto 5 de 2004                                          --
-- Fernando Rengifo                                                 --
----------------------------------------------------------------------

CREATE TABLE gel_datos_modulos_contratos
(
  PRIMARY KEY (id_objeto),
  numero_contratos INTEGER           -- Lmite de contratos
) INHERITS (sis_datos_modulos);


CREATE TABLE gel_dependencias
(
  PRIMARY KEY (id_objeto),
  nombre VARCHAR(255),
  FOREIGN KEY (id_contenedor) REFERENCES gel_datos_modulos_contratos (id_objeto),
  FOREIGN KEY (id_modulo) REFERENCES gel_datos_modulos_contratos (id_objeto)
) INHERITS (sis_objetos_protegidos);

CREATE TABLE gel_contratos_pagina
(
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES gel_datos_modulos_contratos (id_objeto)
) INHERITS (sis_paginas);

CREATE TABLE gel_contratos_pagina_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES gel_contratos_pagina (id_objeto)
) INHERITS (sis_paginas_versiones);


CREATE TABLE gel_estados
(
  id_estado SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);

CREATE TABLE gel_categorias_contratos
(
  id_categoria_contrato SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);

CREATE TABLE gel_tipos_contratos
(
  id_tipo_contrato SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);


CREATE TABLE gel_contratos
(
  id_dependencia INT,
  id_estado INT,
  id_tipo_contrato INT,
  id_categoria_contrato INT,
  numero varchar(255),
  nombre varchar(255),
  objeto text,
  contratista varchar(255),
  fecha_adjudicacion date,
  ordenador_gasto varchar(255),
  activo smallint default '1',
  direccion_compra_pliegos  varchar(255),
  horario_atencion  varchar(255),
  precio integer,
  fecha_hora_apertura_cp timestamp,
  fecha_hora_cierre_cp timestamp,
  departamento varchar(255),
  ciudad varchar(255),
  direccion_propuestas varchar(255),
  fecha_hora_apertura_pr timestamp,
  fecha_hora_cierre_pr timestamp,
  direccion_web_pr varchar(255),
  email_pi varchar(100),
  direccion_web_pi varchar(255), 
  unidad_ejecutora varchar(100),
  presupuesto bigint,    
  PRIMARY KEY (id_objeto), 
  FOREIGN KEY (id_estado) REFERENCES gel_estados(id_estado),
  FOREIGN KEY (id_tipo_contrato) REFERENCES gel_tipos_contratos(id_tipo_contrato),  
  FOREIGN KEY (id_categoria_contrato) REFERENCES gel_categorias_contratos(id_categoria_contrato),    
  FOREIGN KEY (id_modulo) REFERENCES gel_datos_modulos_contratos(id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES gel_datos_modulos_contratos(id_objeto),
  FOREIGN KEY (id_dependencia) REFERENCES gel_dependencias (id_objeto)
) INHERITS (sis_paginas);


CREATE TABLE gel_historial_contratos
(
id_historial SERIAL PRIMARY KEY,
id_contrato INT, 
descripcion VARCHAR(255),
fecha timestamp,
  FOREIGN KEY (id_contrato) REFERENCES gel_contratos(id_objeto)
);

CREATE TABLE gel_comentarios (
id_comentario SERIAL PRIMARY KEY,
id_contrato INT, 
nombre VARCHAR(255),
ciudad VARCHAR(255),
telefono VARCHAR(100),
email VARCHAR(100),
ip inet, 
comentario TEXT,
fecha DATE,
numero VARCHAR(100),
  FOREIGN KEY (id_contrato) REFERENCES gel_contratos(id_objeto)
);

CREATE TABLE gel_tipos_archivos
(
  id_tipo_archivo SERIAL PRIMARY KEY,
  nombre VARCHAR(255)
);


CREATE TABLE gel_contratos_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES gel_contratos (id_objeto)
) INHERITS (sis_paginas_versiones);

CREATE TABLE gel_archivos
(
  id_archivo SERIAL PRIMARY KEY,
  id_tipo_archivo smallint,
  nombre VARCHAR(255),
  extension char(3),
  fecha_actualizacion timestamp,
  descargas INT,
  id_versiones INT,
  FOREIGN KEY (id_versiones) REFERENCES gel_contratos_versiones(id_versiones),
  FOREIGN KEY (id_tipo_archivo) REFERENCES gel_tipos_archivos (id_tipo_archivo)  
);

CREATE TABLE gel_proponentes
(
  id_proponente SERIAL PRIMARY KEY,
  id_versiones INT,
  FOREIGN KEY (id_versiones) REFERENCES gel_contratos_versiones(id_versiones),
  nombre VARCHAR(255),
  documento VARCHAR(100),
  representante_legal VARCHAR(255),
  direccion VARCHAR(100),
  telefono VARCHAR(100),
  email VARCHAR(100),
  password VARCHAR(100)
);