---------------------------------------------------------------------
-- Autor: JUNIOR TELLO, juniorts7@yahoo.com, rtello@telesat.com.co --
-- Archivo de creacin de tablas para el paquete cuestionarios     --
-- Fecha: 13 de Mayo de 2004                                       --
---------------------------------------------------------------------

-- Posterior a create_sistema.sql

CREATE TABLE cue_datos_modulos_cuestionarios
(
  num_cues int4 DEFAULT 10,
  num_pregs int4 DEFAULT 30,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo) 
) INHERITS (sis_datos_modulos) ;

CREATE TABLE cue_tipos_preguntas
(
  id_objeto serial NOT NULL,
  id_idioma int4 NOT NULL,
  nombre varchar(255),
  PRIMARY KEY (id_objeto, id_idioma),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) ;

CREATE TABLE cue_cuestionarios
(
  tipo varchar NOT NULL,
  fecha_creacion date DEFAULT now(),
  respuestas bool DEFAULT false,
  PRIMARY KEY (id_objeto)
) INHERITS (sis_paginas);

CREATE TABLE cue_cuestionarios_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (sis_paginas_versiones) ;

CREATE TABLE cue_preguntas
(
  id_pregunta serial NOT NULL,
  id_cuestionario int4,
  tipo varchar,
  numero_preg int2,
  columnas int2,
  filas int2,
  requerida bool DEFAULT 'FALSE',
  PRIMARY KEY (id_pregunta),
  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE cue_preguntas_versiones
(
  texto_post text,
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES cue_preguntas (id_pregunta) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (sis_versiones) ;

CREATE TABLE cue_posibles_respuestas
(
  id_respuesta serial NOT NULL,
  id_pregunta int4,
  tipo varchar,
  numero int2,
  PRIMARY KEY (id_respuesta),
  FOREIGN KEY (id_pregunta) REFERENCES cue_preguntas (id_pregunta) ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE cue_posibles_respuestas_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES cue_posibles_respuestas (id_respuesta) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones) WITH OIDS;

CREATE TABLE cue_respuestas_cuestionarios
(
  id_resp_cue serial NOT NULL,
  id_cuestionario int4,
  fecha timestamp DEFAULT now(),
  PRIMARY KEY (id_resp_cue),
  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE cue_respuestas_preguntas
(
  id_resp_preg serial NOT NULL,
  id_resp_cues int4,
  id_respuesta int4,
  id_pregunta int4,
  texto_resp text,
  PRIMARY KEY (id_resp_preg),
  FOREIGN KEY (id_pregunta) REFERENCES cue_preguntas (id_pregunta) ON UPDATE CASCADE ON DELETE CASCADE
) ;

CREATE TABLE cue_rc_anonimos
(
  ip varchar,
  CONSTRAINT cue_rc_anonimos_pkey PRIMARY KEY (id_resp_cue),
  CONSTRAINT "$1" FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (cue_respuestas_cuestionarios) WITH OIDS;

CREATE TABLE public.cue_rc_contactos_nuevos
(
  id_contacto int4,
  PRIMARY KEY (id_resp_cue),
  FOREIGN KEY (id_cuestionario) REFERENCES cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
  --FOREIGN KEY (id_contacto) REFERENCES public.cnt_contactos_externos (id_contacto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (cue_respuestas_cuestionarios);

CREATE TABLE cue_rc_registrados
(
  id_usuario int4,
  PRIMARY KEY (id_resp_cue),
  FOREIGN KEY (id_cuestionario) REFERENCES public.cue_cuestionarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (id_usuario) REFERENCES public.sis_usuarios (id_objeto) ON UPDATE CASCADE ON DELETE CASCADE
) INHERITS (cue_respuestas_cuestionarios);


------------------------------------------------------------------
-----------------------  TRIGGERS  -------------------------------
------------------------------------------------------------------

CREATE  OR REPLACE FUNCTION cue_preguntas__cue_tipos_preguntas() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila cue_tipos_preguntas%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM cue_tipos_preguntas 
	     WHERE id_objeto = NEW.tipo;
	
	     IF fila.id_objeto ISNULL THEN
	     RAISE EXCEPTION ''Error CMSII: de integridad referencial de cue_preguntas en cue_tipos_preguntas'';
	     END IF;
		
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_preguntas__cue_tipos_preguntas
BEFORE INSERT OR UPDATE ON cue_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_preguntas__cue_tipos_preguntas('cue_preguntas', 'tipo', 'cue_tipos_preguntas', 'id_objeto');

------------------------------------------------------------------

CREATE  OR REPLACE FUNCTION cue_respuestas_preguntas__cue_respuestas_cuestionarios() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila cue_respuestas_cuestionarios%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM cue_respuestas_cuestionarios 
	     WHERE id_resp_cue = NEW.id_resp_cues;
		
	     IF fila.id_resp_cue ISNULL THEN
	     RAISE EXCEPTION ''Error CMSII: de integridad referencial de cue_respuestas_preguntas en cue_respuestas_cuestionarios'';
	     END IF;
		
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_respuestas_preguntas__cue_respuestas_cuestionarios
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cue_respuestas_cuestionarios('cue_respuestas_preguntas', 'id_resp_cues', 'cue_respuestas_cuestionarios', 'id_resp_cue');

------------------------------------------------------------------

CREATE  OR REPLACE FUNCTION cue_respuestas_preguntas__cuestionarios_hijos() 
RETURNS TRIGGER AS '
     BEGIN	
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_respuestas_preguntas__cue_rc_anonimos
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cuestionarios_hijos('cue_respuestas_preguntas', 'id_resp_cues', 'cue_rc_anonimos', 'id_resp_cue');


CREATE TRIGGER cue_respuestas_preguntas__cue_rc_registrados
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cuestionarios_hijos('cue_respuestas_preguntas', 'id_resp_cues', 'cue_rc_registrados', 'id_resp_cue');


CREATE TRIGGER cue_respuestas_preguntas__cue_rc_contactos_nuevos
BEFORE INSERT OR UPDATE ON cue_respuestas_preguntas
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_respuestas_preguntas__cuestionarios_hijos('cue_respuestas_preguntas', 'id_resp_cues', 'cue_rc_contactos_nuevos', 'id_resp_cue');

------------------------------------------------------------------

-- Cuando borre en cue_rc_contactos_nuevos
-- borrar tambien en cue_respuestas_preguntas


CREATE  OR REPLACE FUNCTION cue_rc_contactos_nuevos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_cue cue_respuestas_preguntas%ROWTYPE;
     BEGIN

		DELETE
		FROM cue_respuestas_preguntas
		WHERE id_resp_cues = OLD.id_resp_cue;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_rc_contactos_nuevos__delete
BEFORE DELETE ON cue_rc_contactos_nuevos
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_rc_contactos_nuevos__delete();

------------------------------------------------------------------
-- Cuando borre en cue_rc_registrados
-- borrar tambien en cue_respuestas_preguntas


CREATE  OR REPLACE FUNCTION cue_rc_registrados__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_cue cue_respuestas_preguntas%ROWTYPE;
     BEGIN

		DELETE
		FROM cue_respuestas_preguntas
		WHERE id_resp_cues = OLD.id_resp_cue;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER cue_rc_registrados__delete
BEFORE DELETE ON cue_rc_registrados
FOR EACH ROW 
EXECUTE PROCEDURE 
cue_rc_registrados__delete();

--  THE END  --