<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4.1.2                                                    |
// +----------------------------------------------------------------------+
// | Copyright (c) 2004 Equipo de desarrollo Emedia - Telesat             |
// +----------------------------------------------------------------------+
// | Authors: Enrique Eder <ejeder@telesat.com.co>                        |
// |   	      Maria Luisa Manyoma <mlmanyoma@telesat.com.co>              |
// |          John A. Vargas <jhonvargas@telesat.com.co>                  |
// |          Camilo Torres <ccordoba@telesat.com.co>                     |
// |          Junior Tello <rtello@telesat.com.co>                        |
// +----------------------------------------------------------------------+
//
//
// Clase Fachada que maneja el paquete directorio
//

		/**** MODELO ****/
require_once PATH_DIRECTORIO . 'modelo/Dir_datos_modulos_directorio.php';
require_once PATH_DIRECTORIO . 'modelo/Dir_cargos.php';
require_once PATH_DIRECTORIO . 'modelo/Dir_departamentos.php';
require_once PATH_DIRECTORIO . 'modelo/Dir_contactos.php';
require_once PATH_DIRECTORIO . 'modelo/Dir_directorio.php';
require_once PATH_DIRECTORIO . 'modelo/Dir_directorio_versiones.php';
require_once PATH_DIRECTORIO . 'modelo/Generador_directorio.php';

		/**** CONTROLADOR ****/
// Mapeamiento
include(PATH_DIRECTORIO . 'controlador/mappings.php');

// Formas
require_once PATH_DIRECTORIO . 'controlador/forms/validar_relacionForm.php';

// Acciones
require_once PATH_DIRECTORIO . 'controlador/actions/adicionar_areaAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/adicionar_cargoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/listar_areasAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/cambiar_nombreAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/borrar_areasAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/listar_cargosAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/borrar_cargosAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/adicionar_contactoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/buscarAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/escribir_mailAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/enviar_emailAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/ver_organigramaAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/asignar_relacionAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/quitar_hijoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/modificar_contactoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/mostrar_contactoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/borrar_contactoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/mostrar_cargoAction.php';
require_once PATH_DIRECTORIO . 'controlador/actions/modificar_cargoAction.php';

/**
 * Esta clase resive los mensajes de la accin a ejecutar por
 * medio del controlador, es quien interactua con todas las clases
 * entidades (Modelo) del paquete directorio.
 * 
 *
 * @package  Directorio
 * @version  1.0
 * @author   John A. Vargas <jhonvargas@telesat.com.co>
 * @since    PHP 4.1.2
 */

class Fachada_Directorio
{ 

    /**
 	 * Adiciona un area al directorio (Departamento)
     *
     * @param string $nombre Nombre del area a adicionar
     * @return array Arreglo con los datos del area adicionada
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function adicionar_area($nombre, $id_sitio)
	 {
	 	$datos = new Dir_datos_modulos_directorio;
	 	$datos->get('id_contenedor', $id_sitio);

		$depto = new Dir_departamentos;
		$depto->setnombre($nombre);
		$depto->setid_modulo($datos->getid_objeto());
		$mensaje = ($depto->insert()) ? 'El area se adicion con xito' : 'Error al adicionar el area';
		Util::log('Directorio', 'adicionar_area', $mensaje);
		
        return $depto->toArray();
	 }

    /**
 	 * Adiciona un cargo al directorio.
     *
     * @param string $nombre Nombre del area a adicionar
     * @return array Arreglo con los datos del area adicionada
	 * @author  John A. Vargas
     * @access public
	 * @static
     */
	 function adicionar_cargo($id_sitio, $nombre, $descripcion, $manual)
	 {
	 	$sitio = new Sis_sitios;
		$sitio->get($id_sitio);
		$datos = new Dir_datos_modulos_directorio;
	 	$datos->get('id_contenedor', $id_sitio);	
	 	
		$cargo = new Dir_cargos;
		$cargo->nombre = $nombre;
		$cargo->descripcion = $descripcion;
		$cargo->id_depto = $id_depto;
		$cargo->id_modulo = $datos->id_objeto;
		
		// Copiar manual de funciones
        $path_images = "imagenes_cont/" . $sitio->url . "/";
		if ($manual['name'] != "") {
			$name_file = time() . $manual['name'];
	        if (copy($manual['tmp_name'], $path_images . $name_file)) {
	            $cargo->manual = $name_file; 
	        }
		}
		$id = $cargo->insert();
		$mensaje = ($id) ? 'El cargo se adicion con xito' : 'Error al adicionar el cargo';
		Util::log('Directorio', 'adicionar_cargo', $mensaje);
		
		return $id;
	 } 
	 
    /**
 	 * Modificar un cargo del directorio.
     *
     * @param int $id_sitio Identificador del cargo a modificar
	 *        string $nombre Nombre del cargo
	 *        string $descripcion Descripcion del cargo
	 *        archivo Manual de funciones del cargo
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function modificar_cargo($id_sitio, $nombre, $descripcion, $manual, $id_cargo)
	 {
	 	$sitio = Sis_sitios::staticGet($id_sitio);
		$datos = new Dir_datos_modulos_directorio;
	 	$datos->get('id_contenedor', $id_sitio);	
	 	
		$cargo = new Dir_cargos;
		$cargo->get($id_cargo);
		$cargo->setnombre($nombre);
		$cargo->setdescripcion($descripcion);
		$cargo->setid_depto($id_depto);
		$cargo->setid_modulo($datos->getid_objeto());

		// Copiar manual de funciones
        $path_images = "imagenes_cont/" . $sitio->url . "/";
		if ($manual['name'] != "") {
			$name_file = time() . $manual['name'];
	        if (copy($manual['tmp_name'], $path_images . $name_file)) {
	            $cargo->setmanual($name_file); 
	        }
		}
		
		$mensaje = ($cargo->update()) ? 'El cargo se modific con xito' : 'Error al modificar el cargo';
		Util::log('Directorio', 'modificar_cargo', $mensaje);
		
		Generador_Directorio::generar_directorio();

        return ;
	 } 
	 
    /**
 	 * Obtiene la lista de areas (Departamentos)
     *
     * @param int $id_sitio Identificador del sitio
	 *
     * @return array Arreglo con las areas del sitio
	 * @author  John A. Vargas
     * @access public
	 * @static
     */
	 function listar_areas($id_sitio)          
	 {
	 	$datos = new Dir_datos_modulos_directorio;
		$areas = new Dir_departamentos;
		
		$datos->id_contenedor = $id_sitio;
		$areas->joinAdd($datos);
		$areas->selectAs();
		$areas->orderBy("nombre"); 
		$areas->find();
		
		$lista_areas = new ArrayList();
		while ($areas->fetch())
			$lista_areas->add($areas->toArray());
			
		return $lista_areas->toArray();
	 }
	 
    /**
 	 * Cambia el nombre de un area (Departamento)
     *
     * @param string $nombre_area Nuevo nombre del area
	 *        int $id_area Identificador del area a cambiar
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */     
	 function cambiar_nombre($nombre_area, $id_area)
	 {
		$area = new Dir_departamentos;
		$area->get($id_area);
		
		$area->nombre = $nombre_area;
		
		$mensaje = ($area->update()) ? 'El area se modific con xito' : 'Error al modificar el area';
		Util::log('Directorio', 'modificar_area', $mensaje);
		
		Generador_Directorio::generar_directorio();
	 }

    /**
 	 * Elimina una o varias areas
     *
     * @param array $lista_areas Arreglo con el listado de areas a borrar
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function borrar_areas($lista_areas)
	 {
	 	$mensaje = '';
	 
	 	foreach ($lista_areas as $id_area => $v) {
			$area = new Dir_departamentos;
			$area->get($id_area);

			$contactos = new Dir_contactos;
			$contactos->id_depto = $id_area;
			if ($contactos->count() == 0) {
				$area->delete(); 
				$mensaje = 'El area ' . $area->nombre . ' se elimin con xito';
			} else {
				$mensaje .= '* El area ' . $area->nombre . ' tiene empleados, no puede ser eliminada\\n';
			}
			Util::log('Directorio', 'borrar_areas', $mensaje);
		}

		return $mensaje;
	 }   

    /**
 	 * Obtiene la lista de cargos 
     *
     * @param int $id_sitio Identificador del sitio
	 *
     * @return array Arreglo con las areas del sitio
	 * @author  John A. Vargas
     * @access public
	 * @static
     */       
	 function listar_cargos($id_sitio)          
	 {
	 	$datos = new Dir_datos_modulos_directorio;
		$areas = new Dir_departamentos;
		$cargos = new Dir_cargos;
		
		$datos->setid_contenedor($id_sitio);
		$cargos->joinAdd($datos);
		$cargos->selectAs();
		$cargos->orderBy("nombre");
		$cargos->find();
		
		$lista_cargos = new ArrayList();
		while ($cargos->fetch())
			$lista_cargos->add($cargos->toArray());

		return $lista_cargos->toArray();
	 }

    /**
 	 * Elimina una o varios cargos
     *
     * @param array $lista_cargos Arreglo con el listado de cargos a borrar
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function borrar_cargos($lista_cargos)
	 {
	 	$mensaje = '';
	 	foreach ($lista_cargos as $k => $v) {
			$cargo = new Dir_cargos;
			$cargo->get($k);
			
			$contactos = new Dir_contactos;
			$contactos->id_cargo = $k;
			
			if ($contactos->count() == 0) {
				$cargo->delete();
				$mensaje .= '* El cargo ' . $cargo->nombre . ' se elimin con xito ';
			} else {
				$mensaje .= '* El cargo ' . $cargo->nombre . ' tiene empleados, no puede ser eliminada\\n';
			}
			Util::log('Directorio', 'borrar_cargos', $mensaje);
		}

		return $mensaje;
	 }  

    /**
 	 * Obtiene los datos de un cargo
     *
     * @param int $id_cargo Identificador del cargo a obtener
	 * @return array Los datos del cargo obtenido
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function mostrar_cargo($id_cargo)
	 {
	 	$cargo = new Dir_cargos;
		$cargo->get($id_cargo);
		
		return $cargo->toArray();
	 }  
	 
    /**
 	 * Adiciona un nuevo contacto
     *
     * @param string $nombre Nombre del contacto
	 *        string $apellidos apellido del contacto
	 *        string $fecha_nacimiento Fecha de nacimiento del contacto
	 *        string $email Email del contacto
	 *        string $celular Celular del contacto
	 *        string $telefono1 Telefono1 del contacto
	 *        string $telefono2 Telefono2 del contacto
	 *        char $sexo Sexo del contacto
	 *        int $id_cargo Identificador del cargo del contacto
	 *        string $fehca_ingreso Fecha de ingreso del contacto
	 * @return int Identificador de contacto del nuevo usuario
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function adicionar_contacto($nombre, $apellido1, $apellido2, $fecha_nacimiento, $email,
	 							 $celular, $telefono1, $telefono2, $sexo, $id_cargo,
								 $fecha_ingreso, $id_depto, $foto)
	 {
	 	if ($fecha_nacimiento == '') $fecha_nacimiento = null;
	 	if ($fecha_ingreso == '') $fecha_ingreso = null;
	 
	 	$contacto = new Dir_contactos;
		$contacto->setnombres($nombre);
		$contacto->setapellido1($apellido1);
		$contacto->setapellido2($apellido2);
		$contacto->setfecha_nacimiento($fecha_nacimiento);
		$contacto->setemail($email);
		$contacto->setcelular($celular);
		$contacto->setsexo($sexo);
		$contacto->settelefono1($telefono1);
		$contacto->settelefono2($telefono2);
		$contacto->setid_cargo($id_cargo);
		$contacto->setfecha_ingreso($fecha_ingreso);
		$contacto->setid_depto($id_depto);

		// Gaurdar la foto del empleado
		$dir_imagenes = "imagenes_cont/" . $_SESSION['sitio']['url'] . "/directorio/";

		if (!file_exists($dir_imagenes)) {
			mkdir($dir_imagenes, 0755);
		}

		$name_image = time() . '_' . $foto['name'];
		$path_image = $dir_imagenes . $name_image;
    
	    if(Imagen::copiar_imagen($foto, $path_image) == 1 && $foto['name'] != '')
			$contacto->foto = $name_image;
			
		$id_cnt = $contacto->insert();
		$mensaje = ($id_cnt) ? 'El contacto se adicion con xito' : 'Error al adicionar el contacto';
		Util::log('Directorio', 'Adicionar Contacto', $mensaje);
		
		Generador_Directorio::generar_directorio();

		return $id_cnt;
	 }
	 
    /**
 	 * Buscar usuarios para el email
     *
     * @param string $nombre Nombre del contacto
	 *        string $apellido apellido del contacto
	 *        string $email Email del contacto
	 *        int $id_cargo Identificador del cargo del contacto
	 *        string $fehca_ingreso Fecha de ingreso del contacto
	 *        int $from Nmero del registro desde el cual mostrar los resultados.
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function buscar($nombre, $apellido, $email, $id_area, $fecha_ingreso, $from)
	 {
	 	$contacto = new Dir_contactos;
		
		if ($nombre != '') {
			$contacto->whereAdd("(LOWER(nombres) LIKE LOWER('%$nombre%')");
			$contacto->whereAdd("LOWER(apellido1) LIKE LOWER('%$nombre%'))",'OR');
		}

		if ($apellido != '') 
			$contacto->whereAdd("LOWER(apellido1) LIKE LOWER('%$apellido%')");
	
		if ($email != '')
			$contacto->whereAdd("LOWER(email) LIKE LOWER('%$email%')");
		
		if ($fecha_ingreso != '')
			$contacto->whereAdd(" fecha_ingreso >= '$fecha_ingreso'");
	
		if ($id_area != '') {
			$contacto->whereAdd(" id_depto = '$id_area' ");
		}
			
		$contacto->orderBy("nombres");
		$contacto->find(); 
		
		// Paginar los resultados de bsqueda
		if ($from == '') $from = 0;
		$data = DB_Pager::getData($from, NUM_REGISTROS, $contacto->count(), MAXPAGES);

		$contacto->limit($from, NUM_REGISTROS);
		$contacto->orderBy('nombres'); 
		$contacto->find();
		
		$lista_contactos = new ArrayList();
		while ($contacto->fetch()) {
			// buscar si existe el dato de usuario para el empleado
			$datos = $contacto->toArray();
			$usuario = new Sis_usuarios;
			$usuario->setid_contacto($contacto->getid_contacto());
			
			if ($usuario->find(true)) {
				$data_user = array('login' => $usuario->getnombre(),
								   'id_usuario' => $usuario->getid_objeto());
				$datos = array_merge($datos, $data_user);
			}
				
			$lista_contactos->add($datos);
		}
			
		return array("lista" => $lista_contactos->toArray(), "data" => $data);
	 }
	 
    /**
 	 * Traer el email del administrador del sitio
     *
     * @param int $id_usuario Identificador del usuario logueado
	 *
	 * @return string Email del administrador del sitio
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function traer_email_usuario($id_contacto)
	 {
	 	$contacto = new Sis_contactos;
		$contacto->get($id_contacto);
		$nombre = $contacto->getnombres() . ' ' . $contacto->getapellido1();
		
		return array('email' => $contacto->getemail(), 'nombre' => $nombre);
	 }

    /**
 	 * Envia un mail a los seleccionados
     *
     * @param string $recipientes Listado de correos a quienes se les envia
	 *        string $name_from Nombre de quien envia el mail
	 *        string $email_from Email de quien envia el mail
	 *        string $asunto Asunto del correo
	 *        string $mensaje Mensaje del correo
	 *        string $adjunto Archivo adjunto a enviar
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	function enviar_mail($recipientes, $name_from, $email_from,
						 $asunto, $mensaje, $adjunto)
	{
		// Cabecera del mensaje
		$headers['From']    = "$name_from <$email_from>";
		$headers['Subject'] = $asunto;
		
		// Mime para adicionar multipart (Archivos adjuntos)
		$mime = new Mail_mime();
		$mime->setTXTBody($mensaje);
		$mime->addAttachment($adjunto['tmp_name'], $adjunto['type'], $adjunto['name']);
		
		// Se obtiene el cuerpo del correo a enviar y su cabecera
		$body = $mime->get();
		$hdrs = $mime->headers($headers);

		// Se envia el mail.
		$mail =& Mail::factory('mail');
		return $mail->send($recipientes, $hdrs, $body);
	}
	 
    /**
 	 * Obtiene las relaciones de personal del contacto
     *
     * @param int $id_contacto Identificador del contacto
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function traer_relaciones($id_contacto)
	 {
	 	$contacto = Dir_contactos::staticGet($id_contacto);
		$padre = new Dir_contactos;
		$padre->get($contacto->getid_contacto_padre());
	
		$hijos = new Dir_contactos;
		$hijos->setid_contacto_padre($contacto->getid_contacto());
		$hijos->find();
		
		$lista_hijos = new ArrayList();
		while ($hijos->fetch())
			$lista_hijos->add($hijos->toArray());

		return array('padre' => $padre->toArray(), 
			         'hijos' => $lista_hijos->toArray(),
					 'contacto' => $contacto->toArray());	
	 }
	 
    /**
 	 * Valida que no existan relaciones aciclicas
	 * (El hijo no puede ser descendiente del padre)
     *
     * @param int $id_contacto Identificador del contacto
	 *        int $id_pariente Identificador del pariente
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function validar_relacion($id_contacto, $id_pariente)
	 {	 
	 	if ($id_contacto == $id_pariente) return false;
	
	 	$contacto_h = new Dir_contactos;
		$contacto_h->get($id_contacto);
		
		// Validar que no sea ancestro
		$contacto_p = new Dir_contactos;
		while ($contacto_p->get($id_pariente)) {
			if ($contacto_p->id_contacto_padre == $id_contacto){
				return false;
			}
			$contacto_p = new Dir_contactos;
			$id_pariente = $contacto_p->id_contacto_padre;
		}	
		
		return Fachada_Directorio::validar_hijo($id_pariente, $id_contacto);		
	 }
	 
    /**
 	 * Valida que no existan relaciones aciclicas
	 * (El hijo no puede ser descendiente del padre)
     *
     * @param int $id_contacto Identificador del contacto
	 *        int $id_pariente Identificador del pariente
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function validar_hijo($id_contacto, $id_pariente)
	 {
	 	$contacto = new Dir_contactos;
		$contacto->get($id_contacto);		
		
		$valido = true;
		if ($contacto->id_contacto_padre == $id_pariente) { 
			return false;
		}
			
		$contacto_h = new Dir_contactos;
		$contacto_h->id_contacto_padre = $id_pariente;
		$contacto_h->find();
		while ($contacto_h->fetch())
			$valido = Fachada_Directorio::validar_hijo($id_contacto, $contacto_h->id_contacto);
			
		return $valido;
	 }
	 
    /**
 	 * Asigna relaciones a un contacto
     *
     * @param int $id_contacto Identificador del contacto
	 *        int $id_pariente Identificador del pariente
	 *        string $relacion Tipo de relacion que tiene el pariente con el contacto
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function asignar_relacion($id_contacto, $id_pariente, $relacion)
	 {	 
	 	$contacto = new Dir_contactos;
	 
	 	if ($relacion == "padre") {
			$contacto->get($id_contacto);
			$contacto->id_contacto_padre = $id_pariente;
			$contacto->update();
		} else {
			$contacto->get($id_pariente);
			$contacto->id_contacto_padre = $id_contacto;
			$contacto->update();
		}
		Util::log('Directorio', 'asignar_relacion', 'Se asigno la relacion');
		
		Generador_Directorio::generar_directorio();
	 }
	 
    /**
 	 * Quitar relacin hijo
     *
     * @param int $id_hijo Identificador del hijo
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function quitar_relacion($id_hijo)
	 {
	 	$contacto = new Dir_contactos;
	 	$contacto->get($id_hijo);
		$contacto->setid_contacto_padre('NULL');
		$contacto->update();
		
		Generador_Directorio::generar_directorio();
	 }
	 
    /**
 	 * Muestra la informacion del contacto
     *
     * @param int $id_contacto Identificador del contacto a mostrar
	 *
	 * @return array Arreglo asociativo con los datos del contacto
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function mostrar_contacto($id_contacto)
	 {
	 	$contacto = new Dir_contactos;
		 $contacto->get($id_contacto); 
		 $area = Dir_departamentos::staticGet($contacto->id_depto);
		 $cargo = Dir_cargos::staticGet($contacto->id_cargo);
		
		 $datos = array_merge($contacto->toArray(),
		 					  array('c_cargo' => $cargo->nombre),
							  array('d_area' => $area->nombre));
		 		 
		 
		$usuario = new Sis_usuarios;
		$usuario->id_contacto = $contacto->id_contacto;

		if ($usuario->find(true)) {
			$data_user = array('login' => $usuario->getnombre(),
							   'id_usuario' => $usuario->getid_objeto());
			$datos = array_merge($datos, $data_user);
		}
				
		 return $datos;
	 }
	 
    /**
 	 * Borrar contacto
     *
     * @param int $id_contacto Identificador del contacto a mostrar
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function borrar_contacto($id_contacto)
	 {
	 	$contacto = Dir_contactos::staticGet($id_contacto);	
		$mensaje = ($contacto->delete()) ? 'Se elimino el contacto' : 'Error al elminar el contacto';
		Util::log('Directorio', 'borrar_contacto', $mensaje);
		
		Generador_Directorio::generar_directorio();
		
		return $contacto->toArray();
	 }
	 
    /**
 	 * Actualiza un contacto
     *
     * @param string $nombre Nombre del contacto
	 *        string $apellidos apellido del contacto
	 *        string $fecha_nacimiento Fecha de nacimiento del contacto
	 *        string $email Email del contacto
	 *        string $celular Celular del contacto
	 *        string $telefono1 Telefono1 del contacto
	 *        string $telefono2 Telefono2 del contacto
	 *        char $sexo Sexo del contacto
	 *        int $id_cargo Identificador del cargo del contacto
	 *        string $fehca_ingreso Fecha de ingreso del contacto
	 *
	 * @author  John A. Vargas
     * @access public
	 * @static
     */ 
	 function modificar_contacto($id_contacto, $nombre, $apellido1, $apellido2, $fecha_nacimiento, $email,
	 							 $celular, $telefono1, $telefono2, $sexo, $id_cargo,
								 $fecha_ingreso, $id_depto, $foto)
	 { 
	 	if ($fecha_nacimiento == '') $fecha_nacimiento = null;
	 	if ($fecha_ingreso == '') $fecha_ingreso = null;
		
		$contacto = new Dir_contactos;
	 	$contacto->get($id_contacto);
		$contacto->nombres = $nombre;
		$contacto->apellido1 = $apellido1;
		$contacto->apellido2 = $apellido2;
		$contacto->fecha_nacimiento = $fecha_nacimiento;
		$contacto->email = $email;
		$contacto->celular = $celular;
		$contacto->sexo = $sexo;
		$contacto->telefono1 = $telefono1;
		$contacto->telefono2 = $telefono2;
		$contacto->id_cargo = $id_cargo;
		$contacto->fecha_ingreso = $fecha_ingreso;
		$contacto->id_depto = $id_depto;
		
		// Guardar la foto del empleado
		$dir_imagenes = "imagenes_cont/" . $_SESSION['sitio']['url'] . "/directorio/";
		
		if (!file_exists("imagenes_cont/" . $_SESSION['sitio']['url'])) {
			mkdir("imagenes_cont/" . $_SESSION['sitio']['url'], 0755);
		}
		
		if (!file_exists($dir_imagenes)) {
			mkdir($dir_imagenes, 0755);
		}

		$name_image = time() . '_' . $foto['name'];
		$path_image = $dir_imagenes . $name_image;
		
	    if($foto['name'] != '' && Imagen::copiar_imagen($foto, $path_image) == 1) {
			if ($contacto->getfoto() != '') @unlink($path_image.'/'.$contacto->getfoto());
			$contacto->setfoto($name_image);
		} 
		$id_cnt = $contacto->update(); 
		$mensaje = ($id_cnt) ? 'Se modifico el contacto' : 'Error al modificar el contacto';
		Util::log('Directorio', 'modificar_contacto', $mensaje);
		 
		Generador_Directorio::generar_directorio();

		return $id_cnt;
	 }
}
?>
