---------------------------------------------------------
-- Creacion de las tablas para el modulo de directorio --
-- Fecha: 1 de Abril de 2004                           --
---------------------------------------------------------

-- Modulo de directorio
CREATE TABLE dir_datos_modulos_directorio
(

   num_empleados INTEGER,
   PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos);

-- Departamentos (Areas)
CREATE TABLE dir_departamentos 
(
  id_depto SERIAL,
  nombre VARCHAR(255),
  id_modulo INTEGER,
  PRIMARY KEY (id_depto),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio(id_objeto)
);

-- Cargos
CREATE TABLE dir_cargos
(
  id_cargo SERIAL,
  nombre VARCHAR(255),
  manual VARCHAR(255),
  descripcion TEXT,
  id_modulo INTEGER,
  PRIMARY KEY (id_cargo),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio (id_objeto) ON DELETE CASCADE
);

-- Empleados
CREATE TABLE dir_contactos
(
  id_contacto_padre INTEGER,
  id_cargo INTEGER REFERENCES dir_cargos(id_cargo),
  fecha_ingreso DATE,
  fecha_nacimiento DATE,
  foto VARCHAR(255),
  id_modulo INTEGER,
  id_depto INTEGER,
  PRIMARY KEY (id_contacto),
  FOREIGN KEY (id_contacto_padre) REFERENCES dir_contactos (id_contacto),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio (id_objeto) ON DELETE CASCADE,
  FOREIGN KEY (id_cargo) REFERENCES dir_cargos (id_cargo),
  FOREIGN KEY (id_depto) REFERENCES dir_departamentos (id_depto)
  
) INHERITS (sis_contactos);

-- Representacion de la pagina de directorio
CREATE TABLE dir_directorio
(
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_modulo) REFERENCES dir_datos_modulos_directorio (id_objeto) ON DELETE CASCADE
) INHERITS (sis_paginas);

-- Version en idiomas de la pagina de directorio
CREATE TABLE dir_directorio_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_objeto) REFERENCES dir_directorio (id_objeto) ON DELETE CASCADE,
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones);

--
-- TRIGGERS Para llaves foraneas de clases abstractas
--

--
-- Todo lo que se inserte en dir_datos_modulos_directorio
-- El atributo id_contenedor debe estar como id_objeto en sis_sitios
--

CREATE  OR REPLACE FUNCTION modulo_directorio__sis_sitios() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila sis_sitios%ROWTYPE;
     BEGIN
	     SELECT * INTO fila 
	     FROM sis_sitios 
	     WHERE id_objeto = NEW.id_contenedor;
	
	     IF fila.id_objeto ISNULL THEN
	     RAISE EXCEPTION ''Error CMSII: de integridad referencial de datos_modulos directorio en sis_sitios'';
	     END IF;
		
	     RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_modulo_directorio_sis_sitios
BEFORE INSERT OR UPDATE ON dir_datos_modulos_directorio
FOR EACH ROW 
EXECUTE PROCEDURE 
modulo_directorio__sis_sitios('dir_datos_modulos_directorio', 'id_contenedor', 'sis_sitios', 'id_objeto');
