<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4.1.2                                                    |
// +----------------------------------------------------------------------+
// | Copyright (c) 2004 Equipo de desarrollo Emedia - Telesat             |
// +----------------------------------------------------------------------+
// | Authors: Enrique Eder <ejeder@telesat.com.co>                        |
// |   	      Maria Luisa Manyoma <mlmanyoma@telesat.com.co>              |
// |          John A. Vargas <jhonvargas@telesat.com.co>                  |
// |          Camilo Torres <ccordoba@telesat.com.co>                     |
// |          Junior Tello <rtello@telesat.com.co>                        |
// +----------------------------------------------------------------------+
//

/**
 * Esta clase representa un estilo del mdulo de diseo
 * 
 * @package  Diseno
 * @version  1.0
 * @author   John A. Vargas <jhonvargas@telesat.com.co>
 * @since    PHP 4.1.2
 */
class Estilos
{
    /**
 	 * Lista los estilos del sistema y los definidos por el usuario
     *
     * @param int $id_sitio Identificador del sitio
     * @return array Arreglo con las hojas de estilo
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function listar_hojas_estilos($id_sitio)
	 {
	 	$hoja_estilos = new Sis_hojas_estilos;
	 	$hoja_estilos->id_contenedor = $id_sitio;
		$hoja_estilos->find();
	
		$lista_hojas_estilos = new ArrayList;
		while ($hoja_estilos->fetch())
			$lista_hojas_estilos->add($hoja_estilos->toArray());

        return $lista_hojas_estilos->toArray();
	 }
    
    /**
 	 * Permite crear un listado con la informacion de los estilos
	 * dde una hoja de estilos dada
     *
	 * @param int $id_hoja_estilos Identificador del objeto contenedor
	 *
     * @return array $estilos Arreglo con los estilos difinidos
	 * @author  John A. Vargas
     * @access  public
	 * @static
     */   
	 function listar_estilos($id_hoja_estilos)
	 {	
		$lista_estilos = array();	 
		
		$obj = new Sis_objetos_protegidos;
		$obj->get($id_hoja_estilos);
			
		$hoja_estilos = new Sis_hojas_estilos;
		$hoja_estilos->get($id_hoja_estilos);

		// Atributos que nos son estilos
		$atr_basicos = array_keys(get_object_vars ($obj));
		$mis_estilos = get_object_vars ($hoja_estilos);
	
		foreach ($mis_estilos as $k => $v) 
			if (!in_array($k, $atr_basicos)) { 
				$estilo = new Sis_estilos_textos;
				$estilo->get($hoja_estilos->$k); 
				$lista_estilos[$k] = $estilo->toArray();
			}
		
		return $lista_estilos;
	 }
    
    /**
 	 * Permite la hacer una vista previa del estilo definido
     *
     * @param array $estilos Arreglo con los estilos difinidos
     *
	 * @author  John A. Vargas
     * @access  public
	 * @static
     */   
	 function ver_estilo($id_hoja_estilos)
	{		
		$template_estilo = PATH_DISENO . 'vistas/templates/tpl_estilo.css';
		$contenido_CSS = "";
		
		$lista_estilos = Estilos::listar_estilos($id_hoja_estilos);

		foreach ($lista_estilos as $nombre_estilo => $estilo) { 
			
			$nombre_estilo =  $nombre_estilo;
			foreach ($estilo as $k => $v) ${$k} = $v; 
			if ($tamagno != "") $tamagno .= "px"; 
			if ($borde != "") $tamagno .= "px"; 
			
			// Se obtiene la plantilla para el estilo
			$contenido_estilo = implode("\n",file($template_estilo)); 
			
			// Se evalua la cadena (asignacion de valores del estilo)
			eval("\$contenido_CSS .= \"$contenido_estilo\"; "); 
			foreach ($estilo as $k => $v) ${$k} = ""; 
		}

		return $contenido_CSS;
	}
	
    /**
 	 * Modificar el estilo de la hoja de estilos
     *
     * @param int $id_sitio Identificador del sitio
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function modificar_estilos($id_hoja_estilos, $nombre_hoja_estilo, $id_estilo,
							   $fuente, $color_fondo, $tamagno,	$color_fuente, $borde, $nombre_estilo)
	 {
	 	$hoja_estilos = new Sis_hojas_estilos;
	 	$estilo = new Sis_estilos_textos;
		$hoja_estilos->get($id_hoja_estilos);
		
		if ($id_estilo != '') {
			$estilo->get($id_estilo); 
			$metodo = 'update';
		} else {
			$metodo = 'insert';
		}
		
		// Modificando el estilo
		$estilo->fuente = $fuente;
		$estilo->color_fondo = $color_fondo;
		$estilo->tamagno = $tamagno;
		$estilo->color_fuente = $color_fuente;
		$estilo->borde = $borde;

		if ($nombre_estilo != '') {
			$id = $estilo->$metodo();
			$hoja_estilos->$nombre_estilo = $id;
		}
	
		// Modificando la hoja de estilo
		$hoja_estilos->nombre = $nombre_hoja_estilo;
		$hoja_estilos->update(); 
	 }
    
    /**
 	 * Crea una nueva hoja de estilos
     *
     * @param string $name Nombre de la hoja de estilos
	 * @param int $id_estilo_base Identificador de la hoja de estilos base
     *
	 * @return  int Identificador de la hoja de estilos nueva
	 * @author  John A. Vargas
     * @access  public
	 * @static
     */   
	 function crear_hoja_estilo($nombre, $id_estilo_base)
	{ 
		$hoja_estilos = new Sis_hojas_estilos;
		$hoja_estilos->id_contenedor = $_SESSION['sitio']['id_objeto'];
		
		$estilos = Estilos::listar_estilos($id_estilo_base);
		
		foreach ($estilos as $nombre_estilo => $estilo) {
			$obj_estilo = new Sis_estilos_textos;
			foreach ($estilo as $k => $v) {
				// Se crea cada estilo
				if ($k != 'id_estilo_texto')
					$obj_estilo->$k = $v; 	
			}
			// Se adiciona a la hoja de estilos
			$hoja_estilos->$nombre_estilo = $obj_estilo->insert();
		}
		$hoja_estilos->nombre = $nombre;
		
		$id = $hoja_estilos->insert();

		return $id;
	}
}
?>