<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4.1.2                                                    |
// +----------------------------------------------------------------------+
// | Copyright (c) 2004 Equipo de desarrollo Emedia - Telesat             |
// +----------------------------------------------------------------------+
// | Authors: Enrique Eder <ejeder@telesat.com.co>                        |
// |   	      Maria Luisa Manyoma <mlmanyoma@telesat.com.co>              |
// |          John A. Vargas <jhonvargas@telesat.com.co>                  |
// |          Camilo Torres <ccordoba@telesat.com.co>                     |
// |          Junior Tello <rtello@telesat.com.co>                        |
// +----------------------------------------------------------------------+
//

	/** Componentes de una plantilla **/ 
require_once PATH_DISENO . 'modelo/Flash.php';
require_once PATH_DISENO . 'modelo/Html.php';

/**
 * Esta clase representa una plantilla del mdulo de diseo
 * 
 * @package  Diseno
 * @version  1.0
 * @author   John A. Vargas <jhonvargas@telesat.com.co>
 * @since    PHP 4.1.2
 */
class Plantillas
{	
    /**
 	 * Lista las plantillas del sistema y los definidos por el usuario
     *
     * @param int $id_sitio Identificador del sitio
     * @return array Arreglo con las plantillas
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function listar_plantillas($id_sitio)
	 {
	 	$datos = new Dis_datos_modulo_diseo;
		$datos->get('id_contenedor', $id_sitio);
	 
	 	$plantillas = new Dis_plantillas;
	 	$plantillas->id_contenedor = $datos->id_objeto;
		$plantillas->find();
	
		$lista_plantillas = new ArrayList;
		while ($plantillas->fetch())
			$lista_plantillas->add($plantillas->toArray());

        return $lista_plantillas->toArray();
	 }
	 
    /**
 	 * Asigna una hoja de estilos a una plantilla
     *
     * @param int $id_plantilla Identificador de la plantilla
     * @param int $id_hoja_estilo Identificador de la hoja de estilos
     * @return array Arreglo con las plantillas
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function asignar_hoja_estilo($id_plantilla, $id_hoja_estilo)
	 {
		$plantilla = new Dis_plantillas;
		$plantilla->get($id_plantilla);
		$plantilla->id_estilo = $id_hoja_estilo;
		$plantilla->update();

        return;
	 }
	 
    /**
 	 * Mostrar componentes
     *
     * @param int $id_plantilla Identificador de la plantilla
     * @return array Arreglo con las plantillas
	 * @author  John A. Vargas
     * @access public
	 * @static
     */    
	 function mostrar_componentes($id_plantilla)
	 {
	 	$componente = new Dis_componentes;
		$componente->id_contenedor = $id_plantilla;
		$componente->find();
		
		$plantilla = new Plantillas;
		
		$lista_componentes = new ArrayList;
		$i = 0;
		$contenido_HTML = '';
		while ($componente->fetch()) {
			// Obtenemos el nombre de la clase de cada componente
			$tipo = get_class($componente->get_hijo());
			$clase = ucwords( substr($tipo, strrpos($tipo, '_') + 1) );
			if ($clase != 'Contenido') {
				$metodo = 'mostrar_' . $tipo;
				$obj = new $clase;
				
				// Almacenamos en contenido_HTML el template de cada componente
				$obj->$metodo($componente->id_componente, &$contenido_HTML, $i++);
				$lista_componentes->add($componente->toArray());
			}
		}

		$_SESSION['contenido_HTML'] = $contenido_HTML;
		return $lista_componentes->toArray();
	 }

    /**
 	 * Llena los componentes de una plantilla 
     *
	 * @param int $id_plantilla Identificador de la plantilla
	 * @param array $componentes Listado de los componentes a guardar
	 * @author  John A. Vargas <jhonvargas@telesat.com.co>
     * @access  public
	 * @static
     */  
	function llenar_plantilla($id_plantilla, $componentes)
	{ 
		foreach ($componentes as $tipo => $datos) {
			// Guardamos cada componente de acuerdo a su clase
			$clase = ucwords($tipo);
			$obj = new $clase; 
			$obj->guardar($datos);
		}
	}

    /**
 	 * Generar la plantilla
     *
	 * @param int $id_plantilla Identificador de la plantilla
	 * @author  John A. Vargas <jhonvargas@telesat.com.co>
     * @access  public
	 * @static
     */  
	 function generar($id_plantilla, $temp = false)
	 {
		$template_head = PATH_DISENO . "vistas/templates/" . "tpl_head.html";
		$template_foot = PATH_DISENO . "vistas/templates/" . "tpl_foot.html";
		$path = ($temp) ? 'tmp/' : 'html/' . $_SESSION['sitio']['url'] . '/vistas/';
		$archivo_head = ($temp) ? $path . time() . 'head_public.php' : $path . $_SESSION['sis_idioma']['enctype'] . '/head_public.php';
		$archivo_foot =  ($temp) ? $path . time() . 'foot_public.php' : $path . 'foot_public.php';
		
		// Escribimos el head general
		File::writeLine($archivo_head,  implode("\n",file($template_head)), FILE_MODE_WRITE);
		
		// Generar cada componente contenido en la plantilla
		$componente = new Dis_componentes;
		$componente->id_contenedor = $id_plantilla;
		$componente->orderBy('posicion');
		$componente->find();
		$archivo = $archivo_head;
		
		// Se escribe el contenido del bloque
		while ($componente->fetch()) {
			// Obtenemos el nombre de la clase de cada componente
			$tipo = get_class($componente->get_hijo());
			$clase = ucwords( substr($tipo, strrpos($tipo, '_') + 1) );
			if ($clase != 'Contenido') {
				$obj = new $clase;
				$obj->generar($componente->id_componente, $archivo);
			} else {
				$archivo = $archivo_foot;
				File::writeLine($archivo_foot, '', FILE_MODE_WRITE);	
			}
		}
	 
		// Escribimos el head general
		File::writeLine($archivo_foot, '</body></html>');
		
		if ($temp) {
			$archivo = 'tmp/' . time() . 'preview.php'; 
			$include = "<? define ('PATH_IMAGES' , '../imagenes_cont/sitio/');
						   include('../$archivo_head'); 
						   include('../$archivo_foot'); ?>";
			File::writeLine($archivo, $include);
			return $archivo;
		}
		
		return '';		
	 }
}
?>