--------------------------------------------------------------------

-- Archivo de creacin de tablas para el paquete de Diseo  

-- Fecha: Julio 10 de 2004                                              

-- Ultima modificacin: Julio 10 de 2004                          

-- Autor: John A. Vargas                                                

--------------------------------------------------------------------



----------------------------------

-- Clases Objeto del Diseo  

----------------------------------



-- Modulo Diseo

CREATE TABLE dis_datos_modulo_diseo

(

    num_plantillas INTEGER,

    num_estilos INTEGER,

    PRIMARY KEY (id_objeto)

) INHERITS (sis_datos_modulos);

COMMENT ON TABLE dis_datos_modulo_diseo IS 'Agrupacin de los objetos para el mdulo de diseso';   



-- Un componente grfico

CREATE TABLE dis_componentes

(

    id_componente SERIAL,

    id_contenedor INTEGER,

    nombre VARCHAR(255),

	posicion INTEGER,

    PRIMARY KEY (id_componente)

);

COMMENT ON TABLE dis_componentes IS 'Componente grfico que usa una pgina para mostrar su contenido';



-- Paneles (contenedores de componentes)

CREATE TABLE dis_paneles

(

    margenes INTEGER,

	tipo CHAR(1),

    PRIMARY KEY (id_componente)

) INHERITS (dis_componentes);

COMMENT ON TABLE dis_paneles IS 'Es un contenedor de componentes grficos';



-- Paneles que se escriben como frames

CREATE TABLE dis_paneles_frame

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_paneles);

COMMENT ON TABLE dis_paneles_frame IS 'Panel que se interpreta por medio de frames';



-- Paneles que se escriben como tablas

CREATE TABLE dis_paneles_tabla

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_paneles);

COMMENT ON TABLE dis_paneles_tabla IS 'Panel que se interpreta por medio de tablas';





-- Plantilla a aplicar

CREATE TABLE dis_plantillas

(

    id_estilo INTEGER,	

    PRIMARY KEY (id_componente),

    FOREIGN KEY (id_estilo) REFERENCES sis_hojas_estilos (id_objeto)

) INHERITS (dis_paneles);    

COMMENT ON TABLE dis_plantillas IS 'Plantilla aplicada a una o varias pginas';



-- Componente Menu

CREATE TABLE dis_menu 

(

    id_menu INTEGER, 

    PRIMARY KEY (id_componente),

    FOREIGN KEY (id_menu) REFERENCES men_menus (id_objeto)

) INHERITS (dis_componentes) ;

COMMENT ON TABLE dis_menu IS 'Componente grfico de un menu';



-- Menu tipo imagen

CREATE TABLE dis_menu_imagen

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_menu) ;

COMMENT ON TABLE dis_menu_imagen IS 'Componente grfico de un menu con imagenes';



-- Menu tipo flash

CREATE TABLE dis_menu_flash

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_menu) ;

COMMENT ON TABLE dis_menu_flash IS 'Componente grfico de un menu hecho en flash';



-- Menu tipo html

CREATE TABLE dis_menu_html

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_menu) ;

COMMENT ON TABLE dis_menu_html IS 'Componente grfico de un menu con estilos html';



-- Flash

CREATE TABLE dis_flash

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_componentes) ;

COMMENT ON TABLE dis_flash IS 'Componente visual de Flash';



-- Imagen

CREATE TABLE dis_imagen

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_componentes) ;

COMMENT ON TABLE dis_imagen IS 'Componente visual de imagen';



-- Contenido

CREATE TABLE dis_contenido

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_componentes) ;

COMMENT ON TABLE dis_componentes IS 'Componente para el contenido';




-- Html

CREATE TABLE dis_html

(

    PRIMARY KEY (id_componente)

) INHERITS (dis_componentes) ;

COMMENT ON TABLE dis_html IS 'Componente visual de html';



-- Versiones de flash

CREATE TABLE dis_flash_versiones

(

	archivo VARCHAR(255),

	ancho INTEGER,

    alto INTEGER,

    PRIMARY KEY (id_versiones)

) INHERITS (sis_versiones) ;

COMMENT ON TABLE dis_flash_versiones IS 'Version en otro idioma del componente flash';



-- Versiones de imagenes

CREATE TABLE dis_imagen_versiones

(

	imagen VARCHAR(255),

    PRIMARY KEY (id_versiones)

) INHERITS (sis_versiones) ;

COMMENT ON TABLE dis_imagen_versiones IS 'Version en otro idioma del componente imagenes';



-- Versiones de html

CREATE TABLE dis_html_versiones

(

	texto_html TEXT,

    PRIMARY KEY (id_versiones)

) INHERITS (sis_versiones) ;

COMMENT ON TABLE dis_html_versiones IS 'Version en otro idioma del componente html';



