----------------------------------------------------------------
-- Archivo de creacin de tablas para el paquete eventos      --
-- Fecha: Marzo 11 de 2004                                    --
----------------------------------------------------------------

-- Posterior a create_sistema.sql

CREATE TABLE eve_datos_modulos_eventos (
	limite_categorias smallint,
	PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos) ;
COMMENT ON TABLE eve_datos_modulos_eventos IS 'Tabla que relaciona el modulo de eventos con el sitio';

CREATE TABLE eve_tipo_lugares (
	PRIMARY KEY (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE eve_tipo_lugares IS 'Tabla para diferentes tipos de lugares : gimnasio, auditorio, etc. Es necesario tener al menos uno antes de crear un lugar';

CREATE TABLE eve_tipo_lugares_versiones (
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_tipo_lugares (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones);
COMMENT ON TABLE eve_tipo_lugares_versiones IS 'Parte traducible de los tipos de lugar en cada idioma disponible para el sitio';

CREATE TABLE eve_lugares (
	direccion text,
	tipo integer,
	telefono1 character varying(255),
	telefono2 character varying(255),
	contacto character varying(255),
	email character varying(255),
	pais integer,
	region character varying(255),
	ciudad character varying(255),
	PRIMARY KEY (id_objeto),
	FOREIGN KEY (id_contenedor) REFERENCES eve_datos_modulos_eventos (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE eve_lugares IS 'Lugares donde se realizan los eventos. Es necesario al menos uno antes de crear los eventos';

CREATE TABLE eve_lugares_versiones (
	nombre character varying(255),
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_lugares (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones);
COMMENT ON TABLE eve_lugares_versiones IS 'Parte traducible de los lugares en cada idioma disponible para el sitio';

CREATE TABLE eve_categorias (
	PRIMARY KEY (id_objeto)
) INHERITS (sis_categorias);
COMMENT ON TABLE eve_categorias IS 'Categorias en que son agrupados los eventos';

CREATE TABLE eve_categorias_versiones (
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_categorias (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_paginas_versiones);
COMMENT ON TABLE eve_categorias_versiones IS 'Parte traducible de las categorias en cada idioma disponible para el sitio';

CREATE TABLE eve_eventos (
	prioritario character,
	destacado character,
	activo character,
	email_contacto character varying(255),
	lugar integer,
	imagen_chica character varying(255),
	imagen_grande character varying(255),
	valor character varying(255),
	fecha_inicio date,
	fecha_fin date,
	horario character varying(255),
	url character varying(255),
	telefono character varying(255),
	PRIMARY KEY (id_objeto),
	FOREIGN KEY (id_contenedor) REFERENCES eve_categorias (id_objeto),	
	FOREIGN KEY (lugar) REFERENCES eve_lugares (id_objeto)	
) INHERITS (sis_paginas);
COMMENT ON TABLE eve_eventos IS 'Tabla de todos los eventos existentes';

CREATE TABLE eve_eventos_versiones (
	titulo character varying (255),
	descripcion_larga text,
	PRIMARY KEY (id_objeto, id_idioma),
	FOREIGN KEY (id_objeto) REFERENCES eve_eventos (id_objeto) ON DELETE CASCADE,
	FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
) INHERITS (sis_versiones);
COMMENT ON TABLE eve_eventos_versiones IS 'Parte traducible de los eventos en cada idioma disponible para el sitio';


  --
-- Todo lo que se inserte en eve_categorias
-- El atributo id_contenedor debe estar en eve_datos_modulos_eventos
--  en eve_categorias
--

CREATE  OR REPLACE FUNCTION ref__eve_categorias()
RETURNS TRIGGER AS '
     DECLARE
             fila eve_datos_modulos_eventos%ROWTYPE;
             fila2 eve_categorias%ROWTYPE;
     BEGIN
                SELECT * INTO fila
                FROM eve_datos_modulos_eventos
                WHERE id_objeto = NEW.id_contenedor;

                SELECT * INTO fila2
                FROM eve_categorias
                WHERE id_objeto = NEW.id_contenedor;

                IF (fila.id_objeto ISNULL) AND (fila2.id_objeto ISNULL) THEN
                   RAISE EXCEPTION ''Error CMSII: de integridad referencial de id_contenedor en eve_categorias'';
                END IF;

                RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_eve_categorias__eve_datos_modulos_categorias
BEFORE INSERT OR UPDATE ON eve_categorias
FOR EACH ROW
EXECUTE PROCEDURE
ref__eve_categorias('eve_categorias','id_contenedor','eve_datos_modulos_eventos','id_objeto');

CREATE TRIGGER ref__eve_categorias__eve_categorias
BEFORE INSERT OR UPDATE ON eve_categorias
FOR EACH ROW
EXECUTE PROCEDURE
ref__eve_categorias('eve_categorias','id_contenedor','eve_categorias','id_objeto');

--
-- Borrado
--

CREATE OR REPLACE FUNCTION eve_categorias__delete()
  RETURNS trigger AS
'
     DECLARE
             fila eve_categorias%ROWTYPE;
     BEGIN

                SELECT * INTO fila
                FROM eve_categorias
                WHERE id_contenedor = OLD.id_objeto;

                IF fila.id_objeto THEN
                   RAISE EXCEPTION \'Error de integridad referencial de contenedor en categoria de eventos\';
                        END IF;

                RETURN OLD;
     END;
 '
  LANGUAGE 'plpgsql';

CREATE TRIGGER ref_eve_categorias__eve_categorias_delete
BEFORE DELETE ON eve_categorias
FOR EACH ROW
EXECUTE PROCEDURE
eve_categorias__delete();
