----------------------------------------------------------------
-- Archivo de creacin de tablas para el paquete foto_album   --
-- Fecha: Enero 19 de 2004                                    --
----------------------------------------------------------------

-- Posterior a create_sistema.sql

CREATE TABLE fot_datos_modulos_foto_album (
	limite_categorias smallint,
	limite_colecciones smallint,
	limite_imagenes smallint,
	PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos) ;
COMMENT ON TABLE fot_datos_modulos_foto_album IS 'Tabla que relaciona el modulo con un sitio. Contiene los limites de categorias, colecciones e imagenes que se pueden almacenar.';

CREATE TABLE fot_categorias (
    img_archivo character varying(255),
    img_pie character varying(255),
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_datos_modulos_foto_album(id_objeto)
)INHERITS (sis_categorias);
COMMENT ON TABLE fot_categorias IS 'Categorias de imagenes del sitio';

CREATE TABLE fot_categorias_versiones (
	nombre character varying(255),
	descripcion text,
    PRIMARY KEY (id_objeto, id_idioma),
    FOREIGN KEY (id_objeto) REFERENCES fot_categorias(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_paginas_versiones);
COMMENT ON TABLE fot_categorias_versiones IS 'Parte traducible de las categorias para cada idioma disponible en el sitio';

CREATE TABLE fot_colecciones (
    -- id_contenedor es el id_objeto de la categoria a la que pertenece
    img_archivo character varying(255),
	posicion integer NOT NULL,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_categorias(id_objeto)
)INHERITS (sis_paginas);
COMMENT ON TABLE fot_colecciones IS 'Colecciones en las que estan divididas las categorias';

CREATE TABLE fot_colecciones_versiones (
	nombre character varying(255),
	descripcion text,
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_objeto) REFERENCES fot_colecciones(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_paginas_versiones);
COMMENT ON TABLE fot_colecciones_versiones IS 'Parte traducible de las colecciones para cada idioma disponible en el sitio';

CREATE TABLE fot_imagenes (
    img_archivo character varying(255),
    img_archivo_thumb character varying(255),
    posicion integer,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_colecciones(id_objeto)
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE fot_imagenes IS 'Todas las imagenes del foto album';

CREATE TABLE fot_imagenes_versiones (
	descripcion text,
    PRIMARY KEY (id_objeto, id_idioma),
    FOREIGN KEY (id_objeto) REFERENCES fot_imagenes(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_paginas_versiones);
COMMENT ON TABLE fot_imagenes_versiones IS 'Parte traducible de las imagenes para cada idioma disponible en el sitio';

CREATE TABLE fot_atributos (
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_categorias(id_objeto) ON DELETE CASCADE
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE fot_atributos IS 'Atributos que se le pueden asignar a cada categoria, para que sus valores sean diligenciados en cada coleccion perteneciente a esta';

CREATE TABLE fot_atributos_versiones (
    PRIMARY KEY (id_objeto, id_idioma),
    FOREIGN KEY (id_objeto) REFERENCES fot_atributos(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas(id_idioma)
)INHERITS (sis_versiones);
COMMENT ON TABLE fot_atributos_versiones IS 'Parte traducible de los atributos por cada idioma disponible en el sitio';

CREATE TABLE fot_atributos_valores (
    id_coleccion INTEGER,
    valor TEXT,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES fot_atributos(id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_coleccion) REFERENCES fot_colecciones_versiones(id_versiones) ON DELETE CASCADE
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE fot_atributos_valores IS 'Valores de los atributos de la categoria por cada coleccion. Los valores son asignados a la version de la coleccion';