-----------------------------------------------------------------
-- Archivo de creacin de tablas para el paquete hojas_de_vida --
-- Fecha: Febrero 10 de 2004                                   --
-----------------------------------------------------------------

-- Posterior a create_sistema.sql

CREATE TABLE hdv_datos_modulos_hojas_de_vida (
 PRIMARY KEY (id_objeto)
) INHERITS (sis_datos_modulos);
COMMENT ON TABLE hdv_datos_modulos_hojas_de_vida IS 'Tabla que relaciona el modulo con un sitio';

CREATE TABLE hdv_nivel_educacion(
 PRIMARY KEY(id_objeto)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_nivel_educacion IS 'Son los niveles de educacion que pueden tener los diferentes estudios realizados por una persona';

CREATE TABLE hdv_area_estudio(
 PRIMARY KEY(id_objeto)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_area_estudio IS 'Son las diferentes areas de estudio que puede escoger una persona';

CREATE TABLE hdv_moneda(
  simbolo character varying(10),
  PRIMARY KEY (id_objeto)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_moneda IS 'Tipos de moneda que esten disponibles para el monto del salario. ie: Euro, dollar, entreotros.';

CREATE TABLE hdv_informacion_personal (
 nombre_completo character varying (255),
 apellido_paterno character varying (50),
 apellido_materno character varying (50),
 genero character varying (20),
 tipo_identificacion character varying (50),
 identificacion character varying (50),
 foto character varying (255),
 fecha_nacimiento date,
 direccion character varying (255),
 codigo_postal character varying (50),
 telefono_hogar character varying (50),
 telefono_trabajo character varying (50),
 telefono_movil character varying (50),
 email character varying (50),
 reubicarse_trabajo char(2),
 reubicarse_pais char(2),
 anos_experiencia character varying (50),
 archivo character varying (255),
 publicado char,
 PRIMARY KEY (id_objeto),
 FOREIGN KEY (id_contenedor) REFERENCES hdv_datos_modulos_hojas_de_vida(id_objeto)
)INHERITS (sis_paginas);
COMMENT ON TABLE hdv_informacion_personal IS 'Son los datos personales de la hoja de vida, requiere que exista un registro del usuario en la tabla Sis_contactos';

CREATE TABLE hdv_informacion_personal_versiones(
 profesion character varying (50),
 objetivo_profesional text,
 areas_conocimiento text,
 pais_origen character varying (50),
 nacionalidad character varying (255),
 barrio character varying (150),
 ciudad character varying (150),
 departamento character varying (150),
 pais character varying (50),
 pais_visa1 character varying (50),
 pais_visa2 character varying (50),
 pais_visa3 character varying (50),
 PRIMARY KEY (id_objeto, id_idioma),
 FOREIGN KEY (id_objeto) REFERENCES hdv_informacion_personal (id_objeto),
 FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_informacion_personal_versiones IS 'Parte traducible de la informacion personal por cada idioma disponible para el sitio';

CREATE TABLE hdv_idiomas(
 usuario integer,
 idioma character varying (50),
 nivel character varying (50),
 PRIMARY KEY (id_objeto),
 FOREIGN KEY (idioma) REFERENCES sis_idiomas(id_idioma),
 FOREIGN KEY (id_contenedor) REFERENCES hdv_datos_modulos_hojas_de_vida(id_objeto) 
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE hdv_idiomas IS 'Son los idiomas que domina quien llena la hoja de vida';

CREATE TABLE hdv_estudios_realizados(
 anos_cursados character varying (50),
 ultimo_ano_cursado character varying (50),
 no_terminado char(2),
 PRIMARY KEY (id_objeto),
 FOREIGN KEY (id_contenedor) REFERENCES hdv_datos_modulos_hojas_de_vida(id_objeto) 
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE hdv_estudios_realizados IS 'Estudios realizados por la persona que diligencia la informacion, existe un registro por cada estudio realizado';

CREATE TABLE hdv_estudios_realizados_versiones(
 nivel_educacion character varying (50),
 area_estudio character varying (50),
 institucion character varying (255),
 ciudad character varying (150),
 pais character varying (50),
 PRIMARY KEY (id_objeto, id_idioma),
 FOREIGN KEY (id_objeto) REFERENCES hdv_estudios_realizados (id_objeto),
 FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_estudios_realizados_versiones IS 'Parte traducible de los estudios realizados por cada idioma disponible en el sitio';

CREATE TABLE hdv_expectativas(
 tipo_cargo character varying (50),
 monto character varying (50),
 moneda character varying (50),
 periodo_pago character varying (50),
 forma_pago character varying (50),
 PRIMARY KEY (id_objeto),
 FOREIGN KEY (id_contenedor) REFERENCES hdv_datos_modulos_hojas_de_vida(id_objeto) 
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE hdv_expectativas IS 'Son las expectativas del usuario para su proximo empleo';

CREATE TABLE hdv_expectativas_versiones(
 area_interes1 character varying (50),
 area_interes2 character varying (50),
 area_interes3 character varying (50),
 informacion_adicional text,
 PRIMARY KEY (id_objeto, id_idioma),
 FOREIGN KEY (id_objeto) REFERENCES hdv_expectativas (id_objeto),
 FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_expectativas_versiones IS 'Parte traducible de las expectativas por cada idioma disponible para el sitio';

CREATE TABLE hdv_experiencia_laboral(
 ingreso character varying (50),
 salida character varying (50),
 ejecutivo char(2),
 PRIMARY KEY (id_objeto),
 FOREIGN KEY (id_contenedor) REFERENCES hdv_datos_modulos_hojas_de_vida(id_objeto) 
)INHERITS (sis_objetos_protegidos);
COMMENT ON TABLE hdv_experiencia_laboral IS 'Son las experiencias laborales de la persona, existe un registro por cada experiencia laboral';

CREATE TABLE hdv_experiencia_laboral_versiones(
 empresa character varying (255),
 actividad character varying (50),
 cargo character varying (50),
 funcion character varying (50),
 PRIMARY KEY (id_objeto, id_idioma),
 FOREIGN KEY (id_objeto) REFERENCES hdv_experiencia_laboral (id_objeto),
 FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
)INHERITS (sis_versiones);
COMMENT ON TABLE hdv_experiencia_laboral_versiones IS 'Parte traducible de la experiencia laboral por cada idioma disponible en el sitio';