CREATE OR REPLACE FUNCTION create_data() RETURNS INTEGER AS '
 DECLARE
   id_tipo_modulo INTEGER;
   id_objeto_seq INTEGER;

   id_modulo_ctr INTEGER;
 BEGIN
   SELECT id_tipo INTO id_tipo_modulo 
   FROM sis_tipo_modulo 
   WHERE prefijo = ''ns'';

   INSERT INTO ns_datos_modulos_newsletter (id_contenedor, id_tipo, nombre, descripcion,numero_envios,num_bloques)
   VALUES (1, id_tipo_modulo, ''Newsletter'', ''Manejo de Newsletters'',''NUM_ENVIOS'',''NUM_BLOQUES'');

 
   -- Seleccionar el id que se le asign al datos modulos
   SELECT currval(''sis_objetos_protegidos_id_objeto_seq'') INTO id_objeto_seq;

   INSERT INTO ns_estados (id_estado, nombre) VALUES (1, ''Activo'');
   INSERT INTO ns_estados (id_estado, nombre) VALUES (2, ''Inactivo'');
   INSERT INTO ns_estados (id_estado, nombre) VALUES (3, ''En proceso'');
   INSERT INTO ns_estados (id_estado, nombre) VALUES (4, ''Enviado'');      

  RETURN 1;
 END;
' LANGUAGE 'plpgsql';

SELECT create_data();

DROP FUNCTION create_data();

ALTER TABLE Art_bloques DROP CONSTRAINT "$1";

---------- ASOCIACION: Art_bloques ---> Art_articulos -----------------
-- El id_contenedor en Art_bloques debe ser un id_objeto en Art_articulos
CREATE OR REPLACE FUNCTION ref_bloque_articulo() 
RETURNS TRIGGER AS '
  DECLARE 
	fila art_articulos%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM art_articulos 
	WHERE id_objeto = NEW.id_contenedor;

	IF fila.id_objeto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de bloque en articulos'';
    END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_bloque_articulo 
BEFORE INSERT OR UPDATE ON Art_bloques
FOR EACH ROW 
EXECUTE PROCEDURE ref_bloque_articulo(art_bloques, id_contenedor, art_articulos, id_objeto);

ALTER TABLE Art_articulos_versiones DROP CONSTRAINT "$1";

---------- ASOCIACION: Art_articulos ---> Art_articulos_versiones -----------------
-- El id_objeto en Art_articulos_versiones debe ser un id_objeto en Art_articulos
CREATE OR REPLACE FUNCTION ref_articulo_version() 
RETURNS TRIGGER AS '
  DECLARE 
	fila art_articulos%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM art_articulos 
	WHERE id_objeto = NEW.id_objeto;

	IF fila.id_objeto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de versiones en articulos'';
    END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_articulo_version 
BEFORE INSERT OR UPDATE ON Art_articulos_versiones
FOR EACH ROW 
EXECUTE PROCEDURE ref_articulo_version(Art_articulos_versiones, id_objeto, art_articulos, id_objeto);

---------- ASOCIACION: Ns_enlaces_contactos ---> Sis_contactos -----------------
-- El id_contacto en ns_enlaces_contacto debe ser un id_contacto en Sis_contactos
CREATE OR REPLACE FUNCTION ref_enlace_contacto() 
RETURNS TRIGGER AS '
  DECLARE 
	fila sis_contactos%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM sis_contactos 
	WHERE id_contacto = NEW.id_contacto;

	IF fila.id_contacto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial en Sis_contactos'';
    END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_enlace_contacto
BEFORE INSERT OR UPDATE ON ns_enlaces_contacto
FOR EACH ROW 
EXECUTE PROCEDURE ref_enlace_contacto(ns_enlaces_contacto, id_contacto, sis_contactos, id_contacto);