----------------------------------------------------------------
-- Archivo de creación de tablas para el paquete Productos    --
-- Fecha: Mayo 21 de 2004		                      --
----------------------------------------------------------------

-- Posterior a create_sistema.sql


CREATE TABLE prd_datos_modulos_productos
(
  limite_categorias SMALLINT,
  limite_productos SMALLINT,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo)
) INHERITS (sis_datos_modulos);
COMMENT ON TABLE prd_datos_modulos_productos IS 'Datos generales del modulo del productos del catalogo.';

CREATE TABLE prd_categorias_productos
(
  PRIMARY KEY (id_objeto)
) INHERITS (sis_categorias);
COMMENT ON  TABLE prd_categorias_productos IS 'Categorias de productos del catalogo.';

CREATE TABLE prd_categorias_productos_versiones
(
  imagen varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_categorias_productos (id_objeto)
) INHERITS (sis_paginas_versiones);
COMMENT ON  TABLE  prd_categorias_productos_versiones IS 'Versiones(idiomas) de cada una de las categorias de productos del catalogo.';

CREATE TABLE prd_productos
(
  publicado BOOLEAN,
  precio FLOAT,
  posicion INTEGER,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES prd_categorias_productos (id_objeto)
) INHERITS (sis_paginas);
COMMENT ON  TABLE  prd_productos IS 'Productos del catalogo.';


CREATE TABLE prd_productos_versiones
(
  imagen varchar(255),
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_productos (id_objeto) ON DELETE CASCADE
) INHERITS (sis_paginas_versiones);
COMMENT ON  TABLE  prd_productos IS 'Versiones(idiomas) de cada uno de los productos del catalogo.';

CREATE TABLE prd_atributos
(
  protegido BOOLEAN DEFAULT false,
  tipo varchar(255),
  caracteristica varchar(100),
  PRIMARY KEY (id_objeto)
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  prd_atributos IS 'Nombres de cada uno de los atributos de un  producto o una categoria del catalogo.';
COMMENT ON COLUMN prd_atributos.tipo IS 'Indica si el atributo pertenece a una categoria o un producto'; 
COMMENT ON COLUMN prd_atributos.caracteristica IS 'Indica si el atributo es lista de selección o texto'; 

CREATE TABLE prd_atributos_valores
(
  protegido BOOLEAN DEFAULT false,
  valor text,
  id_producto int4,
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES prd_atributos (id_objeto),
  FOREIGN KEY (id_producto) REFERENCES prd_productos_versiones (id_versiones) ON DELETE CASCADE
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  prd_atributos_valores IS 'Valores que toman cada uno de los atributos.';

CREATE TABLE prd_atributos_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_atributos (id_objeto) ON DELETE CASCADE
) INHERITS (sis_versiones);
COMMENT ON  TABLE  prd_atributos IS 'Versiones de cada atributo.';

CREATE TABLE prd_atributos_opciones
(
  PRIMARY KEY (id_objeto),
  FOREIGN KEY (id_contenedor) REFERENCES prd_atributos (id_objeto) 
) INHERITS (sis_objetos_protegidos);
COMMENT ON  TABLE  prd_atributos IS 'Opciones que tienen los atributos de tipo lista .';


CREATE TABLE prd_atributos_opciones_versiones
(
  PRIMARY KEY (id_versiones),
  FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma),
  FOREIGN KEY (id_objeto) REFERENCES prd_atributos_opciones (id_objeto) ON DELETE CASCADE
) INHERITS (public.sis_versiones) WITH OIDS;
COMMENT ON  TABLE  prd_atributos IS 'Versiones de las opciones que toman los atributos de tipo lista .';



-- 
-- Trigger para categorias en productos
-- Author: John A. Vargas, Maria Luisa Manyoma
-- Date: Abril 02 de 2004

--
-- Todo lo que se inserte en prd_categoria_productos
-- El atributo id_contenedor debe estar en prd_categoria_productos
-- O en prd_datos_modulos_productos
--

CREATE  OR REPLACE FUNCTION prd_categorias_productos__prd_categorias_productos() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
		 fila_dat prd_datos_modulos_productos%ROWTYPE;
     BEGIN
		SELECT * INTO fila_dat 
		FROM prd_datos_modulos_productos
		WHERE id_objeto = NEW.id_contenedor;

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_objeto = NEW.id_contenedor;
	
		IF (fila_art.id_objeto ISNULL) AND (fila_dat.id_objeto ISNULL) THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria Producto'';
			END IF;
		
		RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_categorias_productos__prd_categorias_productos 
BEFORE INSERT OR UPDATE ON prd_categorias_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_categorias_productos__prd_categorias_productos('prd_categorias_productos','id_contenedor','prd_categorias_productos','id_objeto');

-- 
-- Cuando borre en prd_categorias_productos
-- verificar que el id_objeto no este como id_contenedor
-- en prd_categoria_productos

CREATE  OR REPLACE FUNCTION prd_categorias_productos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_categorias_productos__delete
BEFORE DELETE ON prd_categorias_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_categorias_productos__delete();

-- 
-- Cuando borre en prd_datos_modulos_productos
-- verificar que el id_objeto no este como id_contenedor
-- en prd_categoria_productos

CREATE  OR REPLACE FUNCTION prd_datos_modulos_productos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_datos_modulos_productos__delete
BEFORE DELETE ON prd_datos_modulos_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_datos_modulos_productos__delete();