-- 
-- Trigger para categorias en productos
-- Author: John A. Vargas, Maria Luisa Manyoma
-- Date: Abril 02 de 2004

--
-- Todo lo que se inserte en prd_categoria_productos
-- El atributo id_contenedor debe estar en prd_categoria_productos
-- O en prd_datos_modulos_productos
--

CREATE  OR REPLACE FUNCTION prd_categorias_productos__prd_categorias_productos() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
		 fila_dat prd_datos_modulos_productos%ROWTYPE;
     BEGIN
		SELECT * INTO fila_dat 
		FROM prd_datos_modulos_productos
		WHERE id_objeto = NEW.id_contenedor;

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_objeto = NEW.id_contenedor;
	
		IF (fila_art.id_objeto ISNULL) AND (fila_dat.id_objeto ISNULL) THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria Producto'';
			END IF;
		
		RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_categorias_productos__prd_categorias_productos 
BEFORE INSERT OR UPDATE ON prd_categorias_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_categorias_productos__prd_categorias_productos('prd_categorias_productos','id_contenedor','prd_categorias_productos','id_objeto');

-- 
-- Cuando borre en prd_categorias_productos
-- verificar que el id_objeto no este como id_contenedor
-- en prd_categoria_productos

CREATE  OR REPLACE FUNCTION prd_categorias_productos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_categorias_productos__delete
BEFORE DELETE ON prd_categorias_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_categorias_productos__delete();

-- 
-- Cuando borre en prd_datos_modulos_productos
-- verificar que el id_objeto no este como id_contenedor
-- en prd_categoria_productos

CREATE  OR REPLACE FUNCTION prd_datos_modulos_productos__delete() 
RETURNS TRIGGER AS '
     DECLARE 
	     fila_art prd_categorias_productos%ROWTYPE;
     BEGIN

		SELECT * INTO fila_art 
		FROM prd_categorias_productos 
		WHERE id_contenedor = OLD.id_objeto;
	
		IF fila_art.id_objeto  THEN
		   RAISE EXCEPTION ''Error de integridad referencial de contenedor en categoria articulo'';
			END IF;
		
		RETURN OLD;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER ref_prd_datos_modulos_productos__delete
BEFORE DELETE ON prd_datos_modulos_productos
FOR EACH ROW 
EXECUTE PROCEDURE 
prd_datos_modulos_productos__delete();