----------------------------------------------------------------
-- Archivo de creacin de tablas para el paquete de Sistemas  --
-- Fecha: Diciembre 15 de 2003                                --
-- Ultima modificacin: Mayo 5 de 2004                        --
-- Autor: John A. Vargas                                      --
----------------------------------------------------------------

-------------------------------
-- Clases Objeto del sistema --
-------------------------------

-- Objetos protegidos
CREATE TABLE sis_objetos_protegidos ( 
    id_objeto SERIAL,
    id_contenedor INTEGER,
    id_modulo INTEGER,
    publico INTEGER DEFAULT 1,
    PRIMARY KEY(id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES sis_objetos_protegidos (id_objeto)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_objetos_protegidos IS 'Todo objeto sobre el cual un usuario debe tener un rol para poder manipularlo. Es una super clase abstracta';   
COMMENT ON COLUMN sis_objetos_protegidos.id_objeto IS 'Identificador nico de un objeto'; 
COMMENT ON COLUMN sis_objetos_protegidos.id_contenedor IS 'Identificador del objeto que contiene a este, representa la jerarquia de contenimiento'; 
COMMENT ON COLUMN sis_objetos_protegidos.id_modulo IS 'Identifica que modulo tiene a este objeto'; 
COMMENT ON COLUMN sis_objetos_protegidos.publico IS 'Dice si el objeto es publico o protegido'; 


-- Tipos de identificacion
CREATE TABLE sis_tipo_identificacion (
    nombre VARCHAR(50) NOT NULL,
    PRIMARY KEY (nombre)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_tipo_identificacion IS 'Tipos de identificacin para un contacto, como cc, ce, cm.';   
COMMENT ON COLUMN sis_tipo_identificacion.nombre IS 'Nombre del tipo de identificacion'; 



-- Contactos Generales
CREATE TABLE sis_contactos (
    id_contacto serial,
    apellido1 VARCHAR(50),
    apellido2 VARCHAR(50),
    tipo_identificacion VARCHAR(50),
    email VARCHAR(50),
    celular VARCHAR(50),
    sexo character(1),
    nombres VARCHAR(100),
    cargo VARCHAR(60),
    telefono1 VARCHAR(50),
    telefono2 VARCHAR(50),
    empresa VARCHAR(150),
    identificacion VARCHAR(50),
    PRIMARY KEY(id_contacto),
    FOREIGN KEY (tipo_identificacion) REFERENCES sis_tipo_identificacion (nombre)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_contactos IS 'Informacin detallada de un usuario; son los datos generales';   


CREATE TABLE sis_usuarios (
    nombre VARCHAR(255),
    password VARCHAR(100),
    pregunta TEXT,
    respuesta TEXT,
    id_contacto INTEGER,
    PRIMARY KEY(id_objeto),
    FOREIGN KEY (id_contacto) REFERENCES sis_contactos (id_contacto)
    -- Asociacion id_contendor ==> TRIGGER (Clase abstracta Sis_sitios)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_usuarios IS 'Usuarios que requieren autenticacion para usar el sistema, tanto como adimistrador como navegante';   


CREATE TABLE sis_grupos (
    nombre VARCHAR(255),
    PRIMARY KEY (id_objeto)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_grupos IS 'Agrupacin de usuarios que comparten permisos sobre objetos';   


CREATE TABLE sis_grupos_usuarios (
    id_usuario INTEGER,
    id_grupo INTEGER,
    FOREIGN KEY (id_usuario) REFERENCES sis_usuarios (id_objeto)  ON DELETE CASCADE,
    FOREIGN KEY (id_grupo) REFERENCES sis_grupos (id_objeto)    
);
-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_grupos_usuarios IS 'Producto cartesiano entre grupos y usuarios (Relacin muchos a muchos)';   


CREATE TABLE sis_roles (
    id_rol SERIAL,
    tipo INTEGER DEFAULT 0,
    id_usuario INTEGER,
    id_objeto INTEGER NOT NULL,
    PRIMARY KEY (id_rol)
    -- Asociacin de id_usuario con el id_grupo de sis_grupos o id_objeto de sis_usuarios ==> TRIGGER (asociacin disyunta)
    -- Asociacion con sis_objetos_protegidos ==> TRIGGER (clase abstracta)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_roles IS 'Permisos de un usuario sobre un objeto'; 


CREATE TABLE sis_estilos_textos (
    id_estilo_texto SERIAL,
    fuente VARCHAR(32),
    color_fuente VARCHAR(32),
    tamagno INTEGER,
    color_fondo VARCHAR(32),
    borde VARCHAR(32),
    PRIMARY KEY (id_estilo_texto)
);

CREATE TABLE sis_hojas_estilos (
    h1 INTEGER,
    h2 INTEGER,
    h3 INTEGER,
    h4 INTEGER,
    body INTEGER,
    a INTEGER,
    th INTEGER,
    tr_par INTEGER,
    tr_impar INTEGER,
    tabla_formulario INTEGER,
    boton INTEGER,
    menu INTEGER,
    menu_over INTEGER,
    cabecera INTEGER,
    nombre VARCHAR(255),
    PRIMARY KEY (id_objeto),
    -- Cada objeto de la hoja de estilos es de tipo estilo texto
    FOREIGN KEY (h1) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (h2) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (h3) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (h4) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (body) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (a) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (th) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (tr_par) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (tr_impar) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (tabla_formulario) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (boton) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (menu) REFERENCES sis_estilos_textos(id_estilo_texto),
    FOREIGN KEY (menu_over) REFERENCES sis_estilos_textos(id_estilo_texto)
) INHERITS (sis_objetos_protegidos);

CREATE TABLE sis_sitios (
    url VARCHAR(50),
    id_hoja_estilo INTEGER,
    num_usuarios INTEGER,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_hoja_estilo) REFERENCES sis_hojas_estilos(id_objeto)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_sitios IS 'Sitio que esta siendo administrado con CMS'; 


CREATE TABLE sis_idiomas (
    id_idioma SERIAL,
    nombre VARCHAR(50),
    bandera VARCHAR(50),
    enctype CHAR(5),
    PRIMARY KEY (id_idioma)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_idiomas IS 'Idiomas en los que se alimenta sitios de CMS'; 


CREATE TABLE sis_estilos_bodys (
    scroll_fondo VARCHAR(32),
    scroll_barra VARCHAR(32),
    scroll_flecha VARCHAR(32),
    PRIMARY KEY (id_estilo_texto)
) INHERITS (sis_estilos_textos);

CREATE TABLE sis_paginas (
    id_estilo INTEGER,
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_estilo) REFERENCES sis_hojas_estilos(id_objeto)
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_paginas IS 'Pagina es todo objeto que es enlazable desde el menu.'; 


-- Tabla para los tipos de modulos
CREATE TABLE sis_tipo_modulo (
    id_tipo SERIAL,
    nombre VARCHAR(50),
    descripcion VARCHAR(255),
    carpeta VARCHAR(50),
    prefijo CHAR(3),
    PRIMARY KEY (id_tipo)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_tipo_modulo IS 'Conjunto de mdulos del sistema. Datos estandares de ellos.'; 


CREATE TABLE sis_datos_modulos (
    id_tipo INTEGER,
	nombre VARCHAR (50),
	descripcion VARCHAR (255),
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_contenedor) REFERENCES sis_sitios (id_objeto), 
    FOREIGN KEY (id_tipo) REFERENCES sis_tipo_modulo (id_tipo) 
) INHERITS (sis_objetos_protegidos);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_datos_modulos IS 'Elemento del conjunto de mdulos. Datos generales de el mdulo que representa.'; 


CREATE TABLE sis_categorias (
    PRIMARY KEY (id_objeto),
    FOREIGN KEY (id_modulo) REFERENCES sis_datos_modulos (id_objeto)  
    -- Asociacion de id_contenedor ==> TRIGGER (categoria  datos_modulo)
) INHERITS (sis_paginas);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_categorias IS 'Clase abstracta para las categorias de objetos de CMS'; 


CREATE TABLE sis_portales (
    PRIMARY KEY (id_objeto)
) INHERITS (sis_sitios);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_portales IS 'Portales del sistema CMS'; 


----------------------------------------------------------------
-- Clases Versiones para los objetos del sistema (Traduccin) --
----------------------------------------------------------------

CREATE TABLE sitio_idioma (
    id_sitio INTEGER,
    id_idioma INTEGER,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) 
  -- Asociacion id_sitio ==> TRIGGER (clase abstracta Sis_sitios)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sitio_idioma IS 'Producto cartesiano entre sitios e idiomas (Relacin muchos a ,muchos)'; 


CREATE TABLE sis_versiones (
    id_versiones SERIAL,  
    id_objeto INTEGER NOT NULL,
    id_idioma INTEGER NOT NULL,
    nombre VARCHAR(255),
    descripcion TEXT,
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_objeto) REFERENCES sis_objetos_protegidos (id_objeto),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_versiones IS 'Versiones de los objetos CMS, traduccin en los idiomas del sitio.'; 


CREATE TABLE sis_paginas_versiones (
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) ON DELETE CASCADE,
    FOREIGN KEY (id_objeto) REFERENCES sis_paginas (id_objeto) ON DELETE CASCADE
) INHERITS (sis_versiones);

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_paginas_versiones IS 'Versiones de los pginas CMS, traduccin en los idiomas del sitio.'; 


CREATE TABLE sis_portal_versiones (
    logotipo VARCHAR(100),
    background VARCHAR(100),
    slogan TEXT,
    nombre_pagina VARCHAR(255),
    metatags TEXT,
    metakeywords TEXT,
    PRIMARY KEY (id_versiones),
    FOREIGN KEY (id_objeto) REFERENCES sis_portales (id_objeto) ON DELETE CASCADE,
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma) ON DELETE CASCADE
) INHERITS (sis_versiones); 

-- Comentar la tabla anterior y sus atributos
COMMENT ON TABLE sis_portal_versiones IS 'Versiones de las cabeceras de un portal, traduccin en los idiomas del sitio.'; 


------------------------------------------------------
-- Datos usados para selecciones en los formularios --
------------------------------------------------------

CREATE TABLE sis_actividades_economicas (
    id_actividad SERIAL,
    nombre VARCHAR(255),
    id_idioma INTEGER,
    PRIMARY KEY (id_actividad),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);

CREATE TABLE sis_paises (
    id_paises SERIAL,
    nombre VARCHAR(50),
    id_idioma INTEGER,
    PRIMARY KEY (id_paises),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);

CREATE TABLE sis_profesiones (
    id_profesion SERIAL,
    nombre VARCHAR(50),
    id_idioma INTEGER,
    PRIMARY KEY (id_profesion),
    FOREIGN KEY (id_idioma) REFERENCES sis_idiomas (id_idioma)
);

------------------------------------------------------------
-- Asociaciones en clases abtractas o bifurcas (TRIGGERS) --
------------------------------------------------------------

---------- ASOCIACION: sis_roles ---> sis_objetos_protegidos -----------------
-- El id_objeto en sis_roles debe ser un id_objeto en  sis_objetos_protegidos
CREATE OR REPLACE FUNCTION ref_rol_objeto() 
RETURNS TRIGGER AS '
  DECLARE 
	fila sis_objetos_protegidos%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM sis_objetos_protegidos 
	WHERE id_objeto = NEW.id_objeto;

	IF fila.id_objeto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de rol en objeto_protegido'';
        END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_rol 
BEFORE INSERT OR UPDATE ON sis_roles
FOR EACH ROW 
EXECUTE PROCEDURE ref_rol_objeto(sis_roles, id_objeto, sis_objetos_protegidos, id_objeto);

------------- ASOCIACION: sis_usuarios ---> sis_sitios ------------------
-- El id_contenedor en sis_usuarios debe ser un id_objeto en  sis_sitios
CREATE OR REPLACE FUNCTION ref_usuarios_sitios() 
RETURNS TRIGGER AS '
  DECLARE 
	fila sis_sitios%ROWTYPE;
  BEGIN
	SELECT * INTO fila 
	FROM sis_sitios
	WHERE id_objeto = NEW.id_contenedor;

	IF fila.id_objeto ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de usuario en sitios'';
        END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_usuario_sitio
BEFORE INSERT OR UPDATE ON sis_usuarios
FOR EACH ROW 
EXECUTE PROCEDURE ref_usuarios_sitios(sis_usuarios, id_contenedor, sis_sitios, id_objeto);

------------- ASOCIACION: sis_roles ---> sis_usuarios || sis_grupos ------------------
-- El id_usuario de sis_roles debe estar como id_objeto de sis_usuarios  id_grupo de sis_grupos
CREATE OR REPLACE FUNCTION ref_roles_usuarios_grupos() 
RETURNS TRIGGER AS '
  DECLARE 
	ident_grupo INTEGER;
	ident_usuario INTEGER;
  BEGIN
	SELECT id_objeto INTO ident_usuario
	FROM sis_usuarios
	WHERE id_objeto = NEW.id_usuario;

	SELECT id_objeto INTO ident_grupo
	FROM sis_grupos
	WHERE id_objeto = NEW.id_usuario;
    
	IF ident_usuario ISNULL AND ident_grupo ISNULL THEN
	   RAISE EXCEPTION ''Error de integridad referencial de rol en usuarios o grupos'';
        END IF;
	
	RETURN NEW;
     END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER referencia_roles_usuarios_grupos
BEFORE INSERT OR UPDATE ON sis_roles
FOR EACH ROW 
EXECUTE PROCEDURE ref_roles_usuarios_grupos(sis_roles, id_usuario, sis_usuarios, id_objeto);

CREATE TRIGGER referencia_roles_usuarios_grupos2
BEFORE INSERT OR UPDATE ON sis_roles
FOR EACH ROW 
EXECUTE PROCEDURE ref_roles_usuarios_grupos(sis_roles, id_usuario, sis_grupos, id_grupo);